@@ -39,6 +39,8 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+/* $Id: acap.c,v 1.29 2002/05/25 19:57:41 leg Exp $ */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -58,8 +60,9 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#include <sasl.h>
+#include <sasl/sasl.h>
 #include "../lib/prot.h" /* xxx should be <cyrus/prot.h> */
+#include "../lib/iptostring.h"
 #include "../lib/xmalloc.h"
 
 #include "acap.h"
@@ -330,7 +333,7 @@ static void acap_auth_step(acap_value_t *arg, void *rock)
 {
     int r;
     acap_conn_t *conn = (acap_conn_t *) rock;
-    char *out;
+    const char *out;
     unsigned int outlen;
 
     /* just got a response from the server; process it and generate our own */
@@ -345,7 +348,6 @@ static void acap_auth_step(acap_value_t *arg, void *rock)
 	} else {
 	    prot_printf(conn->pout, "\"\"\r\n");
 	}
-	if (out) free(out);
     } else {
 	/* cancel authentication */
 	prot_printf(conn->pout, "*\r\n");
@@ -408,10 +410,11 @@ static int acap_conn_do_auth(acap_conn_t *conn,
 			     const char *mechs,
 			     sasl_callback_t *cb)
 {
-    struct sockaddr_in sin;
+    struct sockaddr_in saddr_l, saddr_r;
+    char localip[60], remoteip[60];
     sasl_security_properties_t *secprops=NULL;
     const char *mech;
-    char *str;
+    const char *str;
     socklen_t salen;
     unsigned len;
     int r;
@@ -461,32 +464,29 @@ static int acap_conn_do_auth(acap_conn_t *conn,
     mycb[i].proc = NULL;
     mycb[i].context = NULL;
 
-    r = sasl_client_new("acap", conn->host, mycb, 
-			SASL_SECURITY_LAYER, &conn->sasl);
-    if (r != SASL_OK || !conn->sasl) {
-	conn->sasl_error = r;
-	free(mycb);
-	return ACAP_NO_AUTHENTICATION;
-    }
-
-    salen = sizeof(sin);
-    if (getpeername(conn->sock, (struct sockaddr *) &sin, &salen)) {
+    salen = sizeof(saddr_r);
+    if (getpeername(conn->sock, (struct sockaddr *) &saddr_r, &salen)) {
 	free(mycb);
 	return ACAP_NO_CONNECTION;
     }
-    sasl_setprop(conn->sasl, SASL_IP_REMOTE, &sin);
-    if (r != SASL_OK || !conn->sasl) {
-	conn->sasl_error = r;
-	free(mycb);
-	return ACAP_NO_AUTHENTICATION;
-    }
 
-    salen = sizeof(sin);
-    if (getsockname(conn->sock, (struct sockaddr *) &sin, &salen)) {
+    salen = sizeof(saddr_l);
+    if (getsockname(conn->sock, (struct sockaddr *) &saddr_l, &salen)) {
 	free(mycb);
 	return ACAP_NO_CONNECTION;
     }
-    sasl_setprop(conn->sasl, SASL_IP_LOCAL, &sin);
+
+    if(iptostring((struct sockaddr *)&saddr_l, sizeof(struct sockaddr_in),
+		  localip, 60))
+	return ACAP_NO_CONNECTION;
+
+    if(iptostring((struct sockaddr *)&saddr_r, sizeof(struct sockaddr_in),
+		  remoteip, 60))
+	return ACAP_NO_CONNECTION;
+
+    r = sasl_client_new("acap", conn->host,
+			localip, remoteip,
+			mycb, 0, &conn->sasl);
     if (r != SASL_OK || !conn->sasl) {
 	conn->sasl_error = r;
 	free(mycb);
@@ -500,13 +500,11 @@ static int acap_conn_do_auth(acap_conn_t *conn,
     }
 
     r = sasl_client_start(conn->sasl, mechs,
-			  NULL, NULL,
-			  &str, &len, &mech);
+			  NULL, &str, &len, &mech);
     if (r == SASL_OK || r == SASL_CONTINUE) {
 	if (str && len) {
 	    r = acap_cmd_start(conn, &cmd, 
 			       "Authenticate %s %S", mech, len, str);
-	    free(str);
 	} else {
 	    /* no initial response */
 	    r = acap_cmd_start(conn, &cmd,
