@@ -40,7 +40,7 @@
  *
  */
 /*
- * $Id: annotate.c,v 1.2 2002/04/11 18:27:33 ken3 Exp $
+ * $Id: annotate.c,v 1.8 2002/06/03 15:22:20 ken3 Exp $
  */
 
 #include <config.h>
@@ -164,10 +164,16 @@ void annotatemore_open(char *fname)
 struct fetchdata {
     struct namespace *namespace;
     char *userid;
-    int flags;
+    unsigned entries;
+    unsigned attribs;
     struct entryattlist **entryatts;
 };
 
+enum {
+    ENTRY_SERVER =		(1<<0),
+    ENTRY_PARTITION =		(1<<1)
+};
+
 enum {
     ATTRIB_VALUE =		(1<<0),
     ATTRIB_SIZE =		(1<<1),
@@ -182,7 +188,7 @@ static int fetch_cb(char *name, int matchlen, int maycreate, void* rock)
     static int sawuser = 0;
     int c, r;
     char mboxname[MAX_MAILBOX_PATH+1];
-    char *path, *partition;
+    char *path, *server, *partition;
     char entry[MAX_MAILBOX_PATH+25];
     char size[100];
     struct attvaluelist *attvalues = NULL;
@@ -218,25 +224,53 @@ static int fetch_cb(char *name, int matchlen, int maycreate, void* rock)
 	strcpy(mboxname, name);
 
     /* lookup the partition info */
-    r = mboxlist_detail(mboxname, NULL, &path, &partition, NULL, NULL);
+    r = mboxlist_detail(mboxname, NULL, &path, &server, NULL, NULL);
     if (r) return r;
 
+    partition = strchr(server, '!');
+    if (partition) {
+	*partition++ = '\0';
+    }
+    else {
+	partition = server;
+	server = NULL;
+    }
+
     c = name[matchlen];
     if (c) name[matchlen] = '\0';
     (*fdata->namespace->mboxname_toexternal)(fdata->namespace, name,
 					     fdata->userid, mboxname);
     if (c) name[matchlen] = c;
 
-    sprintf(entry, "/mailbox/{%s}/vendor/cyrus/partition", mboxname);
+    if (server && (fdata->entries & ENTRY_SERVER)) {
+	sprintf(entry, "/mailbox/{%s}/vendor/cmu/cyrus-imapd/server",
+		mboxname);
 
-    if (fdata->flags & ATTRIB_VALUE)
-	appendattvalue(&attvalues, "value.shared", partition);
-    if (fdata->flags & ATTRIB_SIZE) {
-	sprintf(size, "%u", strlen(partition));
-	appendattvalue(&attvalues, "size.shared", size);
+	attvalues = NULL;
+	if (fdata->attribs & ATTRIB_VALUE)
+	    appendattvalue(&attvalues, "value.shared", server);
+	if (fdata->attribs & ATTRIB_SIZE) {
+	    sprintf(size, "%u", strlen(server));
+	    appendattvalue(&attvalues, "size.shared", size);
+	}
+
+	appendentryatt(fdata->entryatts, entry, attvalues);
     }
 
-    appendentryatt(fdata->entryatts, entry, attvalues);
+    if (fdata->entries & ENTRY_PARTITION) {
+	sprintf(entry, "/mailbox/{%s}/vendor/cmu/cyrus-imapd/partition",
+		mboxname);
+
+	attvalues = NULL;
+	if (fdata->attribs & ATTRIB_VALUE)
+	    appendattvalue(&attvalues, "value.shared", partition);
+	if (fdata->attribs & ATTRIB_SIZE) {
+	    sprintf(size, "%u", strlen(partition));
+	    appendattvalue(&attvalues, "size.shared", size);
+	}
+
+	appendentryatt(fdata->entryatts, entry, attvalues);
+    }
 
     return 0;
 }
@@ -247,35 +281,52 @@ int annotatemore_fetch(struct strlist *entries, struct strlist *attribs,
 {
     struct strlist *e = entries;
     struct strlist *a = attribs;
-    char *mailbox, *cp;
+    char *mailbox, *cp, *wildcard;
     struct fetchdata fdata;
 
     *l = NULL;
 
     /* we only do shared annotations right now */
-    fdata.flags = 0;
+    fdata.attribs = 0;
     while (a) {
 	if (!strcmp(a->s, "*") || !strcmp(a->s, "%"))
-	    fdata.flags |= ATTRIB_VALUE | ATTRIB_SIZE;
+	    fdata.attribs |= ATTRIB_VALUE | ATTRIB_SIZE;
 	else if (!strcmp(a->s, "value.*") || !strcmp(a->s, "value.%") ||
 		 !strcmp(a->s, "value") || !strcmp(a->s, "value.shared"))
-	    fdata.flags |= ATTRIB_VALUE;
+	    fdata.attribs |= ATTRIB_VALUE;
 	else if (!strcmp(a->s, "size.*") || !strcmp(a->s, "size.%") ||
 		 !strcmp(a->s, "size") || !strcmp(a->s, "size.shared"))
-	    fdata.flags |= ATTRIB_SIZE;
+	    fdata.attribs |= ATTRIB_SIZE;
 
 	a = a->next;
     }
 
-    if (!fdata.flags) return 0;
+    if (!fdata.attribs) return 0;
 
     while (e) {
+	fdata.entries = 0;
+
 	/* XXX fix this cheesy matching stuff so we support wildcards */
 	if (!strncmp(e->s, "/mailbox/{", 10) &&
 	    ((cp = strchr(e->s + 10, '}')) != NULL)) {
 	    mailbox = e->s + 10;
 	    *cp++ = '\0';
-	    if (!strcmp(cp, "/vendor/cyrus/partition")) {
+
+	    /* we only support "/mailbox/{mbox}/vendor/cmu/cyrus-imapd/server"
+	       and ".../vendor/cmu/cyrus-imapd/partition" right now */
+	    if (!strncmp(cp, "/vendor/cmu/cyrus-imapd/server",
+			 (wildcard = strchr(cp, '*')) ? wildcard - cp : 20)) {
+		fdata.entries |= ENTRY_SERVER;
+	    }
+	    if (!strncmp(cp, "/vendor/cmu/cyrus-imapd/partition",
+			 (wildcard = strchr(cp, '*')) ? wildcard - cp : 23)) {
+		fdata.entries |= ENTRY_PARTITION;
+	    }
+	    if (!strcmp(cp, "/vendor/cmu/cyrus-imapd/%")) {
+		fdata.entries |= (ENTRY_SERVER | ENTRY_PARTITION);
+	    }
+		
+	    if (fdata.entries) {
 		/* Reset state in fetch_cb */
 		fetch_cb(NULL, 0, 0, 0);
 
@@ -290,25 +341,128 @@ int annotatemore_fetch(struct strlist *entries, struct strlist *attribs,
 	    }
 	}
 
+	if (!strncmp(e->s, "/server/", 8)) {
+	    FILE *f;
+	    char filename[1024], buf[1024], size[100], *p;
+	    struct attvaluelist *attvalues;
+
+	    cp = e->s + 8;
+
+	    if (!strncmp(cp, "motd", strcspn(cp, "*%"))) {
+		sprintf(filename, "%s/msg/motd", config_dir);
+		if ((f = fopen(filename, "r")) != NULL) {
+		    fgets(buf, sizeof(buf), f);
+		    fclose(f);
+
+		    if ((p = strchr(buf, '\r'))!=NULL) *p = 0;
+		    if ((p = strchr(buf, '\n'))!=NULL) *p = 0;
+		    /* can't have [ be first char, sigh */
+		    for(p = buf; *p == '['; p++);
+
+		    attvalues = NULL;
+		    if (fdata.attribs & ATTRIB_VALUE)
+			appendattvalue(&attvalues, "value.shared", buf);
+		    if (fdata.attribs & ATTRIB_SIZE) {
+			sprintf(size, "%u", strlen(buf));
+			appendattvalue(&attvalues, "size.shared", size);
+		    }
+
+		    appendentryatt(l, "/server/motd", attvalues);
+		}
+	    }
+	    if (!strncmp(cp, "comment", strcspn(cp, "*%"))) {
+		sprintf(filename, "%s/msg/comment", config_dir);
+		if ((f = fopen(filename, "r")) != NULL) {
+		    fgets(buf, sizeof(buf), f);
+		    fclose(f);
+
+		    if ((p = strchr(buf, '\r'))!=NULL) *p = 0;
+		    if ((p = strchr(buf, '\n'))!=NULL) *p = 0;
+
+		    attvalues = NULL;
+		    if (fdata.attribs & ATTRIB_VALUE)
+			appendattvalue(&attvalues, "value.shared", buf);
+		    if (fdata.attribs & ATTRIB_SIZE) {
+			sprintf(size, "%u", strlen(buf));
+			appendattvalue(&attvalues, "size.shared", size);
+		    }
+
+		    appendentryatt(l, "/server/comment", attvalues);
+		}
+	    }
+	}
+
 	e = e->next;
     }
 
     return 0;
 }
 
-int annotatemore_store(struct entryattlist *l, int isadmin, char *userid,
+static int server_store(char *filename, char *value)
+{
+    FILE *f;
+
+    /* XXX check for failures */
+    if (!strcmp(value, "NIL"))
+	unlink(filename);
+    else {
+	f = fopen(filename, "w");
+	fprintf(f, "%s\n", value);
+	fclose(f);
+    }
+
+    return 0;
+}
+
+int annotatemore_store(struct entryattlist *l, struct namespace *namespace,
+		       int isadmin, char *userid,
 		       struct auth_state *auth_state)
 {
     struct entryattlist *e = l;
+    struct attvaluelist *av;
+    char *value = NULL, *motd = NULL, *comment = NULL;
+    char filename[1024];
+
+    syslog(LOG_INFO, "annotatemore_store");
 
     while (e) {
 	if (strncmp(e->entry, "/server/", 8) &&
 	    strncmp(e->entry, "/mailbox/", 9)) {
 	    return IMAP_ANNOTATION_BADENTRY;
 	}
+
+	av = e->attvalues;
+	while (av) {
+	    if (!strcmp(av->attrib, "value.shared")) {
+		value = av->value;
+		break;
+	    }
+	    else
+		return IMAP_PERMISSION_DENIED;
+
+	    av = av->next;
+	}
+
+	if (value && !strcmp(e->entry, "/server/motd"))
+	    motd = value;
+	else if (value && !strcmp(e->entry, "/server/comment"))
+	    comment = value;
+	else
+	    return IMAP_PERMISSION_DENIED;
+
 	e = e->next;
     }
 
+    /* XXX check for failures -- how to do this atomic? */
+    if (motd) {
+	sprintf(filename, "%s/msg/motd", config_dir);
+	server_store(filename, value);
+    }
+    if (comment) {
+	sprintf(filename, "%s/msg/comment", config_dir);
+	server_store(filename, value);
+    }
+
     return 0;
 }
 
