@@ -38,7 +38,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: ctl_cyrusdb.c,v 1.9 2002/02/24 04:42:15 ken3 Exp $
+ * $Id: ctl_cyrusdb.c,v 1.14 2002/05/29 16:49:14 rjs3 Exp $
  */
 
 #include <config.h>
@@ -116,10 +116,57 @@ void fatal(const char *message, int code)
 void usage(void)
 {
     fprintf(stderr, "ctl_cyrusdb [-C <altconfig>] -c\n");
-    fprintf(stderr, "ctl_cyrusdb [-C <altconfig>] -r\n");
+    fprintf(stderr, "ctl_cyrusdb [-C <altconfig>] -r [-x]\n");
     exit(-1);
 }
 
+/* Callback for use by recover_reserved */
+static int fixmbox(char *name,
+		   int matchlen __attribute__((unused)),
+		   int maycreate __attribute__((unused)),
+		   void *rock __attribute__((unused)))
+{
+    int mbtype;
+    int r;
+    char *path, *part, *acl;
+
+    /* Do an mboxlist_detail on the mailbox */
+    r = mboxlist_detail(name, &mbtype, &path, &part, &acl, NULL);
+
+    /* if it is MBTYPE_RESERVED, unset it & call mboxlist_delete */
+    if(!r && (mbtype & MBTYPE_RESERVE)) {
+	if(!r) {
+	    r = mboxlist_deletemailbox(name, 1, NULL, NULL, 0, 0, 1);
+	    if(r) {
+		/* log the error */
+		syslog(LOG_ERR,
+		       "could not remove reserved mailbox '%s': %s",
+		       name, error_message(r));
+	    } else {
+		syslog(LOG_ERR,
+		       "removed reserved mailbox '%s'",
+		       name);
+	    }
+	}
+    }
+
+    return 0;
+}
+void recover_reserved() 
+{
+    char pattern[2] = { '*', '\0' };
+    
+    mboxlist_init(0);
+    mboxlist_open(NULL);
+
+    /* build a list of mailboxes - we're using internal names here */
+    mboxlist_findall(NULL, pattern, 1, NULL,
+		     NULL, fixmbox, NULL);
+
+    mboxlist_close();
+    mboxlist_done();
+}
+
 
 int main(int argc, char *argv[])
 {
@@ -127,6 +174,7 @@ int main(int argc, char *argv[])
     int opt, r, r2;
     char *alt_config = NULL;
     int flag = 0;
+    int reserve_flag = 1;
     enum { RECOVER, CHECKPOINT, NONE } op = NONE;
     char dirname[1024], backup1[1024], backup2[1024];
     char *archive_files[N(dblist)];
@@ -136,7 +184,7 @@ int main(int argc, char *argv[])
     if (geteuid() == 0) fatal("must run as the Cyrus user", EC_USAGE);
     r = r2 = 0;
 
-    while ((opt = getopt(argc, argv, "C:rc")) != EOF) {
+    while ((opt = getopt(argc, argv, "C:rxc")) != EOF) {
 	switch (opt) {
 	case 'C': /* alt config file */
 	    alt_config = optarg;
@@ -155,6 +203,10 @@ int main(int argc, char *argv[])
 	    else usage();
 	    break;
 
+	case 'x':
+	    reserve_flag = 0;
+	    break;
+
 	default:
 	    usage();
 	    break;
@@ -166,6 +218,11 @@ int main(int argc, char *argv[])
 	exit(1);
     }
 
+    if(op != RECOVER && !reserve_flag) {
+	usage();
+	exit(1);
+    }
+
     config_init(alt_config, "ctl_cyrusdb");
 
     /* create the name of the db directory */
@@ -289,6 +346,9 @@ int main(int argc, char *argv[])
 	}
     }
 
+    if(op == RECOVER && reserve_flag)
+	recover_reserved();
+
     syslog(LOG_NOTICE, "done %s", msg);
     exit(r || r2);
 }
