@@ -1,4 +1,4 @@
-/* $Id: cyrdump.c,v 1.5 2001/03/15 22:56:18 leg Exp $
+/* $Id: cyrdump.c,v 1.9 2002/02/01 19:43:37 rjs3 Exp $
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -70,6 +70,14 @@ static void print_seq(const char *tag, const char *attrib,
 		      unsigned *seq, int n);
 int usage(const char *name);
 
+/* available from elsewhere */
+int index_getuidsequence(struct mailbox *mailbox, 
+                        struct searchargs *searchargs,
+                        unsigned **uid_list);
+
+/* current namespace */
+static struct namespace dump_namespace;
+
 int imapd_exists;
 struct protstream *imapd_out = NULL;
 struct auth_state *imapd_authstate = NULL;
@@ -83,7 +91,7 @@ int main(int argc, char *argv[])
 {
     int option;
     char buf[MAX_MAILBOX_PATH];
-    int i;
+    int i, r;
     char *alt_config = NULL;
     struct incremental_record irec;
 
@@ -115,10 +123,19 @@ int main(int argc, char *argv[])
     mboxlist_init(0);
     mboxlist_open(NULL);
 
+    /* Set namespace -- force standard (internal) */
+    if ((r = mboxname_init_namespace(&dump_namespace, 1)) != 0) {
+	syslog(LOG_ERR, error_message(r));
+	fatal(error_message(r), EC_CONFIG);
+    }
+
     irec.incruid = 0;
     for (i = optind; i < argc; i++) {
 	strlcpy(buf, argv[optind], MAX_MAILBOX_NAME);
-	mboxlist_findall(buf, 1, 0, 0, dump_me, &irec);
+	/* Translate any separators in mailboxname */
+	mboxname_hiersep_tointernal(&dump_namespace, buf);
+	(*dump_namespace.mboxlist_findall)(&dump_namespace, buf, 1, 0, 0,
+					   dump_me, &irec);
     }
 
     mboxlist_close();
@@ -144,7 +161,7 @@ void fatal(const char *s, int code)
 static void generate_boundary(char *boundary)
 {
     snprintf(boundary, 100, "dump-%ld-%ld-%ld", 
-	     getpid(), time(NULL), rand());
+	     (long) getpid(), (long) time(NULL), (long) rand());
 }
 
 static int dump_me(char *name, int matchlen, int maycreate, void *rock)
@@ -191,7 +208,7 @@ static int dump_me(char *name, int matchlen, int maycreate, void *rock)
     printf("\n");
 
     printf("<imapdump uniqueid=\"%s\">\n", m.uniqueid);
-    imapurl_toURL(imapurl, config_servername, m.name);
+    imapurl_toURL(imapurl, config_servername, m.name, NULL);
     printf("  <mailbox-url>%s</mailbox-url>\n", imapurl);
     printf("  <incremental-uid>%d</incremental-uid>\n", irec->incruid);
     printf("  <nextuid>%ld</nextuid>\n", m.last_uid + 1);
