@@ -1,5 +1,5 @@
 /* lmtpengine.c: LMTP protocol engine
- * $Id: lmtpengine.c,v 1.72 2002/04/17 20:31:52 ken3 Exp $
+ * $Id: lmtpengine.c,v 1.75 2002/06/06 00:12:34 rjs3 Exp $
  *
  * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -136,21 +136,6 @@ static struct
     char *authid;
 } saslprops = {NULL,NULL,0,NULL};
 
-/* a simple hash function for sasl mechanisms */
-static int hash_simple (const char *str)
-{
-    int     value = 0;
-    int     i;
-
-    if (!str)
-	return 0;
-    for (i = 0; *str; i++)
-    {
-	value ^= (*str++ << ((i & 3)*8));
-    }
-    return value;
-}
-
 /* round to nearest 1024 bytes and return number of Kbytes.
  used for SNMP updates. */
 static int roundToK(int x)
@@ -1044,7 +1029,10 @@ static int savemsg(struct clientdata *cd,
     r = fill_cache(cd->pin, f, m);
     if (r) {
 	fclose(f);
-	func->removespool(m);
+	if (func->removespool) {
+	    /* remove the spool'd message */
+	    func->removespool(m);
+	}
 	while (nrcpts--) {
 	    send_lmtp_error(cd->pout, r);
 	}
@@ -1821,7 +1809,6 @@ void lmtpmode(struct lmtp_func *func,
 		r=tls_init_serverengine("lmtp",
 					5,   /* depth to verify */
 					1,   /* can client auth? */
-					0,   /* require client to auth? */
 					1);   /* TLS only? */
 
 		if (r == -1) {
