@@ -1,6 +1,6 @@
 /* lmtpproxyd.c -- Program to proxy mail delivery
  *
- * $Id: lmtpproxyd.c,v 1.40 2002/04/30 16:49:28 rjs3 Exp $
+ * $Id: lmtpproxyd.c,v 1.42 2002/06/03 18:22:26 rjs3 Exp $
  * Copyright (c) 1999-2000 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -146,7 +146,7 @@ void shut_down(int code);
 static void usage();
 
 struct lmtp_func mylmtp = { &deliver, &verify_user, &shut_down,
-			    &proxy_spoolfile, 0, 0, 0 };
+			    &proxy_spoolfile, NULL, 0, 0, 0 };
 
 /* globals */
 static int quotaoverride = 0;		/* should i override quota? */
@@ -279,11 +279,12 @@ int service_main(int argc, char **argv, char **envp)
     prot_setflushonread(deliver_in, deliver_out);
     prot_settimeout(deliver_in, 300);
 
-    while ((opt = getopt(argc, argv, "C:q")) != EOF) {
+    while ((opt = getopt(argc, argv, "C:Dq")) != EOF) {
 	switch(opt) {
 	case 'C': /* alt config file - handled by service::main() */
 	    break;
-
+	case 'D': /* ext debugger - handled by service::main() */
+ 	    break;
 	case 'q':
 	    quotaoverride = 1;
 	    break;
