@@ -1,5 +1,5 @@
 /* mailbox.c -- Mailbox manipulation routines
- $Id: mailbox.c,v 1.125 2002/04/29 20:00:23 rjs3 Exp $
+ $Id: mailbox.c,v 1.134 2002/06/20 16:35:52 rjs3 Exp $
  
  * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -62,6 +62,7 @@
 #include <sys/stat.h>
 #include <ctype.h>
 #include <time.h>
+#include <com_err.h>
 
 #ifdef HAVE_DIRENT_H
 # include <dirent.h>
@@ -329,7 +330,12 @@ int mailbox_open_header_path(const char *name,
 
     mailbox->name = xstrdup(name);
     mailbox->path = xstrdup(path);
+
+    /* Note that the header does have the ACL information, but it is only
+     * a backup, and the mboxlist data is considered authoritative, so
+     * we will just use what we were passed */
     mailbox->acl = xstrdup(acl);
+
     mailbox->myrights = cyrus_acl_myrights(auth_state, mailbox->acl);
 
     if (mailbox->header_fd == -1) return 0;
@@ -899,6 +905,7 @@ struct mailbox *mailbox;
     strcpy(fnamebuf, mailbox->path);
     strcat(fnamebuf, FNAME_INDEX);
 
+    /* xxx why is this not a lock_reopen() ? */
     for (;;) {
 	r = lock_blocking(mailbox->index_fd);
 	if (r == -1) {
@@ -1061,11 +1068,14 @@ struct quota *quota;
 int mailbox_write_header(struct mailbox *mailbox)
 {
     int flag;
-    FILE *newheader;
     int newheader_fd;
+    int r = 0;
+    const char *quota_root;
     char fnamebuf[MAX_MAILBOX_PATH];
     char newfnamebuf[MAX_MAILBOX_PATH];
     struct stat sbuf;
+    struct iovec iov[10];
+    int niov;
 
     assert(mailbox->header_lock_count != 0);
 
@@ -1074,38 +1084,55 @@ int mailbox_write_header(struct mailbox *mailbox)
     strcpy(newfnamebuf, fnamebuf);
     strcat(newfnamebuf, ".NEW");
 
-    newheader = fopen(newfnamebuf, "w+");
-    if (!newheader) {
+    newheader_fd = open(newfnamebuf, O_CREAT | O_TRUNC | O_RDWR, 0666);
+    if (newheader_fd == -1) {
 	syslog(LOG_ERR, "IOERROR: writing %s: %m", newfnamebuf);
 	return IMAP_IOERROR;
     }
 
-    fputs(MAILBOX_HEADER_MAGIC, newheader);
-    fprintf(newheader, "%s\t%s\n", 
-	    mailbox->quota.root ? mailbox->quota.root : "",
-	    mailbox->uniqueid);
-    for (flag = 0; flag < MAX_USER_FLAGS; flag++) {
-	if (mailbox->flagname[flag]) {
-	    fprintf(newheader, "%s ", mailbox->flagname[flag]);
+    /* Write magic header, do NOT write the trailing NUL */
+    r = write(newheader_fd, MAILBOX_HEADER_MAGIC,
+	      sizeof(MAILBOX_HEADER_MAGIC) - 1);
+
+    if(r != -1) {
+	niov = 0;
+	quota_root = mailbox->quota.root ? mailbox->quota.root : "";
+	WRITEV_ADDSTR_TO_IOVEC(iov,niov,(char *)quota_root);
+	WRITEV_ADD_TO_IOVEC(iov,niov,"\t",1);
+	WRITEV_ADDSTR_TO_IOVEC(iov,niov,mailbox->uniqueid);
+	WRITEV_ADD_TO_IOVEC(iov,niov,"\n",1);
+	r = retry_writev(newheader_fd, iov, niov);
+    }
+
+    if(r != -1) {
+	for (flag = 0; flag < MAX_USER_FLAGS; flag++) {
+	    if (mailbox->flagname[flag]) {
+		niov = 0;
+		WRITEV_ADDSTR_TO_IOVEC(iov,niov,mailbox->flagname[flag]);
+		WRITEV_ADD_TO_IOVEC(iov,niov," ",1);
+		r = retry_writev(newheader_fd, iov, niov);
+		if(r == -1) break;
+	    }
 	}
     }
-    fprintf(newheader, "\n");
-    fprintf(newheader, "%s\n", mailbox->acl);
-
-    fflush(newheader);
-    newheader_fd = dup(fileno(newheader));
-    if (ferror(newheader) || fsync(fileno(newheader)) ||
+    
+    if(r != -1) {
+	niov = 0;
+	WRITEV_ADD_TO_IOVEC(iov,niov,"\n",1);
+	WRITEV_ADDSTR_TO_IOVEC(iov,niov,mailbox->acl);
+	WRITEV_ADD_TO_IOVEC(iov,niov,"\n",1);
+	r = retry_writev(newheader_fd, iov, niov);
+    }
+    
+    if (r == -1 || fsync(newheader_fd) ||
 	lock_blocking(newheader_fd) == -1 ||
 	rename(newfnamebuf, fnamebuf) == -1) {
 	syslog(LOG_ERR, "IOERROR: writing %s: %m", newfnamebuf);
-	fclose(newheader);
 	close(newheader_fd);
 	unlink(newfnamebuf);
 	return IMAP_IOERROR;
     }
 
-    fclose(newheader);
-
     if (mailbox->header_fd != -1) {
 	close(mailbox->header_fd);
 	map_free(&mailbox->header_base, &mailbox->header_len);
@@ -1116,10 +1143,11 @@ int mailbox_write_header(struct mailbox *mailbox)
 	syslog(LOG_ERR, "IOERROR: fstating %s: %m", fnamebuf);
 	fatal("can't fstat header file", EC_OSFILE);
     }
+
     map_refresh(mailbox->header_fd, 1, &mailbox->header_base,
 		&mailbox->header_len, sbuf.st_size, "header", mailbox->name);
     mailbox->header_ino = sbuf.st_ino;
-    
+
     return 0;
 }
 
@@ -1292,14 +1320,13 @@ int mailbox_append_index(struct mailbox *mailbox,
 /*
  * Write out the quota 'quota'
  */
-int
-mailbox_write_quota(quota)
-struct quota *quota;
+int mailbox_write_quota(struct quota *quota)
 {
     int r;
+    int len;
+    char buf[1024];
     char quota_path[MAX_MAILBOX_PATH];
     char new_quota_path[MAX_MAILBOX_PATH];
-    FILE *newfile;
     int newfd;
 
     assert(quota->lock_count != 0);
@@ -1310,27 +1337,27 @@ struct quota *quota;
     strcpy(new_quota_path, quota_path);
     strcat(new_quota_path, ".NEW");
 
-    newfile = fopen(new_quota_path, "w+");
-    if (!newfile) {
+    newfd = open(new_quota_path, O_CREAT | O_TRUNC | O_RDWR, 0666);
+    if (newfd == -1) {
 	syslog(LOG_ERR, "IOERROR: creating quota file %s: %m", new_quota_path);
 	return IMAP_IOERROR;
     }
-    newfd = dup(fileno(newfile));
+
     r = lock_blocking(newfd);
     if (r) {
 	syslog(LOG_ERR, "IOERROR: locking quota file %s: %m",
 	       new_quota_path);
-	fclose(newfile);
 	close(newfd);
 	return IMAP_IOERROR;
     }
 
-    fprintf(newfile, "%lu\n%d\n", quota->used, quota->limit);
-    fflush(newfile);
-    if (ferror(newfile) || fsync(fileno(newfile))) {
+    len = snprintf(buf, sizeof(buf) - 1,
+		   "%lu\n%d\n", quota->used, quota->limit);
+    r = write(newfd, buf, len);
+    
+    if (r == -1 || fsync(newfd)) {
 	syslog(LOG_ERR, "IOERROR: writing quota file %s: %m",
 	       new_quota_path);
-	fclose(newfile);
 	close(newfd);
 	return IMAP_IOERROR;
     }
@@ -1338,11 +1365,9 @@ struct quota *quota;
     if (rename(new_quota_path, quota_path)) {
 	syslog(LOG_ERR, "IOERROR: renaming quota file %s: %m",
 	       quota_path);
-	fclose(newfile);
 	close(newfd);
 	return IMAP_IOERROR;
     }
-    fclose(newfile);
 
     if (quota->fd != -1) {
 	close(quota->fd);
@@ -1967,6 +1992,7 @@ int mailbox_create(const char *name,
     struct mailbox mailbox;
     int save_errno;
     int n;
+    const char *lockfailaction;
     struct stat sbuf;
 
     while ((p = strchr(p+1, '/'))) {
@@ -1998,35 +2024,56 @@ int mailbox_create(const char *name,
     strcpy(fnamebuf, path);
     p = fnamebuf + strlen(fnamebuf);
     strcpy(p, FNAME_HEADER);
-    mailbox.header_fd = open(fnamebuf, O_WRONLY|O_TRUNC|O_CREAT, 0666);
+    mailbox.header_fd = open(fnamebuf, O_RDWR|O_TRUNC|O_CREAT, 0666);
     if (mailbox.header_fd == -1) {
 	syslog(LOG_ERR, "IOERROR: creating %s: %m", fnamebuf);
 	return IMAP_IOERROR;
     }
 
+    /* Note that we are locking the mailbox here.  Technically, this function
+     * can be called with a lock on the mailbox list.  This would otherwise
+     * violate the locking semantics, but it is okay since the mailbox list
+     * changes have not been committed, and the mailbox we create here *can't*
+     * be opened by anyone else */
+
+    r = lock_reopen(mailbox.header_fd, fnamebuf, NULL, &lockfailaction);
+    if(r) {
+	syslog(LOG_ERR, "IOERROR: %s header for new mailbox %s: %m",
+	       lockfailaction, mailbox.name);
+	mailbox_close(&mailbox);
+	return IMAP_IOERROR;
+    }
+    mailbox.header_lock_count++;
+
     mailbox.name = xstrdup(name);
     mailbox.path = xstrdup(path);
     mailbox.acl = xstrdup(acl);
 
     strcpy(p, FNAME_INDEX);
-    mailbox.index_fd = open(fnamebuf, O_WRONLY|O_TRUNC|O_CREAT, 0666);
+    mailbox.index_fd = open(fnamebuf, O_RDWR|O_TRUNC|O_CREAT, 0666);
     if (mailbox.index_fd == -1) {
 	syslog(LOG_ERR, "IOERROR: creating %s: %m", fnamebuf);
 	mailbox_close(&mailbox);
 	return IMAP_IOERROR;
     }
 
+    r = lock_reopen(mailbox.index_fd, fnamebuf, NULL, &lockfailaction);
+    if(r) {
+	syslog(LOG_ERR, "IOERROR: %s index for new mailbox %s: %m",
+	       lockfailaction, mailbox.name);
+	mailbox_close(&mailbox);
+	return IMAP_IOERROR;
+    }
+    mailbox.index_lock_count++;
+
     strcpy(p, FNAME_CACHE);
-    mailbox.cache_fd = open(fnamebuf, O_WRONLY|O_TRUNC|O_CREAT, 0666);
+    mailbox.cache_fd = open(fnamebuf, O_RDWR|O_TRUNC|O_CREAT, 0666);
     if (mailbox.cache_fd == -1) {
 	syslog(LOG_ERR, "IOERROR: creating %s: %m", fnamebuf);
 	mailbox_close(&mailbox);
 	return IMAP_IOERROR;
     }
-
-    mailbox.header_lock_count = 1;
-    mailbox.index_lock_count = 1;
-
+        
     if (hasquota) mailbox.quota.root = xstrdup(quota_root);
     mailbox.generation_no = 0;
     mailbox.format = format;
@@ -2075,7 +2122,7 @@ int mailbox_create(const char *name,
 
 /*
  * Delete and close the mailbox 'mailbox'.  Closes 'mailbox' whether
- * or not the deletion was successful.
+ * or not the deletion was successful.  Requires a locked mailbox.
  */
 int mailbox_delete(struct mailbox *mailbox, int delete_quota_root)
 {
@@ -2084,16 +2131,11 @@ int mailbox_delete(struct mailbox *mailbox, int delete_quota_root)
     struct dirent *f;
     char buf[MAX_MAILBOX_PATH];
     char *tail;
+    
+    /* Ensure that we are locked */
+    if(!mailbox->header_lock_count) return IMAP_INTERNAL;
 
-    /* Lock everything in sight */
-    r =  mailbox_lock_header(mailbox);
-    if (!r && mailbox->index_fd == -1) r = mailbox_open_index(mailbox);
-    if (!r) r = mailbox_lock_index(mailbox);
-    if (!r) rquota = mailbox_lock_quota(&mailbox->quota);
-    if (r) {
-	mailbox_close(mailbox);
-	return r;
-    }
+    rquota = mailbox_lock_quota(&mailbox->quota);
 
     seen_delete_mailbox(mailbox);
 
@@ -2162,85 +2204,69 @@ static int expungeall(struct mailbox *mailbox __attribute__((unused)),
 /* if 'isinbox' is set, we perform the funky RENAME INBOX INBOX.old
    semantics, regardless of whether or not the name of the mailbox is
    'user.foo'.*/
-int
-mailbox_rename(const char *oldname, const char *oldpath, const char *oldacl, 
-	       const char *newname, char *newpath, int isinbox, 
-	       bit32 *olduidvalidityp, bit32 *newuidvalidityp,
-	       struct mailbox *mailboxp)
+/* requires a LOCKED oldmailbox pointer */
+int mailbox_rename_copy(struct mailbox *oldmailbox, 
+			const char *newname, char *newpath,
+			bit32 *olduidvalidityp, bit32 *newuidvalidityp,
+			struct mailbox *newmailbox)
 {
-    int r, r2;
-    struct mailbox oldmailbox, newmailbox;
+    int r;
     unsigned int flag, msgno;
     struct index_record record;
     char oldfname[MAX_MAILBOX_PATH], newfname[MAX_MAILBOX_PATH];
     char *oldfnametail, *newfnametail;
 
-    /* Open old mailbox and lock */
-    mailbox_open_header_path(oldname, oldpath, oldacl, 0, &oldmailbox, 0);
-
-    if (oldmailbox.format == MAILBOX_FORMAT_NETNEWS) {
-	mailbox_close(&oldmailbox);
-	return IMAP_MAILBOX_NOTSUPPORTED;
-    }
-
-    r =  mailbox_lock_header(&oldmailbox);
-    if (!r) r = mailbox_open_index(&oldmailbox);
-    if (!r) r = mailbox_lock_index(&oldmailbox);
-    if (r) {
-	mailbox_close(&oldmailbox);
-	return r;
-    }
+    assert(oldmailbox->header_lock_count > 0
+	   && oldmailbox->index_lock_count > 0);
+    
 
     /* Create new mailbox */
     r = mailbox_create(newname, newpath, 
-		       oldmailbox.acl, oldmailbox.uniqueid, oldmailbox.format,
-		       &newmailbox);
-    if (r) {
-	mailbox_close(&oldmailbox);
-	return r;
-    }
-    if (strcmp(oldname, newname) == 0) {
+		       oldmailbox->acl, oldmailbox->uniqueid,
+		       oldmailbox->format, newmailbox);
+
+    if (r) return r;
+
+    if (strcmp(oldmailbox->name, newname) == 0) {
 	/* Just moving mailboxes between partitions */
-	newmailbox.uidvalidity = oldmailbox.uidvalidity;
+	newmailbox->uidvalidity = oldmailbox->uidvalidity;
     }
 
-    if (olduidvalidityp) *olduidvalidityp = oldmailbox.uidvalidity;
-    if (newuidvalidityp) *newuidvalidityp = newmailbox.uidvalidity;
+    if (olduidvalidityp) *olduidvalidityp = oldmailbox->uidvalidity;
+    if (newuidvalidityp) *newuidvalidityp = newmailbox->uidvalidity;
 
     /* Copy flag names */
     for (flag = 0; flag < MAX_USER_FLAGS; flag++) {
-	if (oldmailbox.flagname[flag]) {
-	    newmailbox.flagname[flag] = xstrdup(oldmailbox.flagname[flag]);
+	if (oldmailbox->flagname[flag]) {
+	    newmailbox->flagname[flag] = xstrdup(oldmailbox->flagname[flag]);
 	}
     }
-    r = mailbox_write_header(&newmailbox);
+    r = mailbox_write_header(newmailbox);
     if (r) {
-	mailbox_close(&newmailbox);
-	mailbox_close(&oldmailbox);
+	mailbox_close(newmailbox);
 	return r;
     }
 
     /* Check quota if necessary */
-    if (newmailbox.quota.root) {
-	r = mailbox_lock_quota(&newmailbox.quota);
-	if (!oldmailbox.quota.root ||
-	    strcmp(oldmailbox.quota.root, newmailbox.quota.root) != 0) {
-	    if (!r && newmailbox.quota.limit >= 0 &&
-		newmailbox.quota.used + oldmailbox.quota_mailbox_used >
-		((unsigned) newmailbox.quota.limit * QUOTA_UNITS)) {
+    if (newmailbox->quota.root) {
+	r = mailbox_lock_quota(&(newmailbox->quota));
+	if (!oldmailbox->quota.root ||
+	    strcmp(oldmailbox->quota.root, newmailbox->quota.root) != 0) {
+	    if (!r && newmailbox->quota.limit >= 0 &&
+		newmailbox->quota.used + oldmailbox->quota_mailbox_used >
+		((unsigned) newmailbox->quota.limit * QUOTA_UNITS)) {
 		r = IMAP_QUOTA_EXCEEDED;
 	    }
 	}
 	if (r) {
-	    mailbox_close(&newmailbox);
-	    mailbox_close(&oldmailbox);
+	    mailbox_close(newmailbox);
 	    return r;
 	}
     }
 
-    strcpy(oldfname, oldmailbox.path);
+    strcpy(oldfname, oldmailbox->path);
     oldfnametail = oldfname + strlen(oldfname);
-    strcpy(newfname, newmailbox.path);
+    strcpy(newfname, newmailbox->path);
     newfnametail = newfname + strlen(newfname);
 
     /* Copy over index/cache files */
@@ -2253,77 +2279,87 @@ mailbox_rename(const char *oldname, const char *oldpath, const char *oldacl,
     unlink(newfname);
     if (!r) r = mailbox_copyfile(oldfname, newfname);
     if (r) {
-	mailbox_close(&newmailbox);
-	mailbox_close(&oldmailbox);
+	mailbox_close(newmailbox);
 	return r;
     }
 
     /* Re-open index file and store new uidvalidity  */
-    close(newmailbox.index_fd);
-    newmailbox.index_fd = dup(oldmailbox.index_fd);
-    (void) mailbox_read_index_header(&newmailbox);
-    newmailbox.generation_no = oldmailbox.generation_no;
-    (void) mailbox_write_index_header(&newmailbox);
+    close(newmailbox->index_fd);
+    newmailbox->index_fd = dup(oldmailbox->index_fd);
+    (void) mailbox_read_index_header(newmailbox);
+    newmailbox->generation_no = oldmailbox->generation_no;
+    (void) mailbox_write_index_header(newmailbox);
 
     /* Copy over message files */
     oldfnametail++;
     newfnametail++;
-    for (msgno = 1; msgno <= oldmailbox.exists; msgno++) {
-	r = mailbox_read_index_record(&oldmailbox, msgno, &record);
+    for (msgno = 1; msgno <= oldmailbox->exists; msgno++) {
+	r = mailbox_read_index_record(oldmailbox, msgno, &record);
 	if (r) break;
-	mailbox_message_get_fname(&oldmailbox, record.uid, oldfnametail);
+	mailbox_message_get_fname(oldmailbox, record.uid, oldfnametail);
 	strcpy(newfnametail, oldfnametail);
 	r = mailbox_copyfile(oldfname, newfname);
 	if (r) break;
     }
-    if (!r) r = seen_copy(&oldmailbox, &newmailbox);
+    if (!r) r = seen_copy(oldmailbox, newmailbox);
 
     /* Record new quota usage */
-    if (!r && newmailbox.quota.root) {
-	newmailbox.quota.used += oldmailbox.quota_mailbox_used;
-	r = mailbox_write_quota(&newmailbox.quota);
-	mailbox_unlock_quota(&newmailbox.quota);
+    if (!r && newmailbox->quota.root) {
+	newmailbox->quota.used += oldmailbox->quota_mailbox_used;
+	r = mailbox_write_quota(&(newmailbox->quota));
+	mailbox_unlock_quota(&(newmailbox->quota));
     }
-    if (r) goto fail;
+    if (r) {
+	/* failure and back out */
+	for (msgno = 1; msgno <= oldmailbox->exists; msgno++) {
+	    if (mailbox_read_index_record(oldmailbox, msgno, &record))
+		continue;
+	    mailbox_message_get_fname(oldmailbox, record.uid, newfnametail);
+	    (void) unlink(newfname);
+	}
+    }
+
+    return r;
+}
+
+/* Requires a locked mailbox */
+int mailbox_rename_finish(struct mailbox *oldmailbox,
+			  struct mailbox *newmailbox,
+			  int isinbox) 
+{
+    int r, r2;
 
     if (isinbox) {
 	/* Expunge old mailbox */
-	r = mailbox_expunge(&oldmailbox, 0, expungeall, (char *)0);
-	mailbox_close(&oldmailbox);
+	r = mailbox_expunge(oldmailbox, 0, expungeall, (char *)0);
     } else {
-	r = mailbox_delete(&oldmailbox, 0);
+	r = mailbox_delete(oldmailbox, 0);
     }
 
-    if (r && newmailbox.quota.root) {
-	r2 = mailbox_lock_quota(&newmailbox.quota);
-	newmailbox.quota.used += newmailbox.quota_mailbox_used;
+    if (r && newmailbox->quota.root) {
+	r2 = mailbox_lock_quota(&(newmailbox->quota));
+	newmailbox->quota.used += newmailbox->quota_mailbox_used;
 	if (!r2) {
-	    r2 = mailbox_write_quota(&newmailbox.quota);
-	    mailbox_unlock_quota(&newmailbox.quota);
+	    r2 = mailbox_write_quota(&(newmailbox->quota));
+	    mailbox_unlock_quota(&(newmailbox->quota));
 	}
 	if (r2) {
 	    syslog(LOG_ERR,
 	      "LOSTQUOTA: unable to record use of %lu bytes in quota %s",
-		   newmailbox.quota_mailbox_used, newmailbox.quota.root);
+		   newmailbox->quota_mailbox_used, newmailbox->quota.root);
 	}
     }
-    if (r) goto fail;
 
-    if (mailboxp) {
-	*mailboxp = newmailbox;
-    } else {
-	mailbox_close(&newmailbox);
+    if(r) {
+	syslog(LOG_CRIT,
+	       "Rename Failure during mailbox_rename_finish (%s->%s), " \
+	       "potential leaked space (%s)", oldmailbox->name,
+	       newmailbox->name, error_message(r));
     }
-    return 0;
 
- fail:
-    for (msgno = 1; msgno <= oldmailbox.exists; msgno++) {
-	if (mailbox_read_index_record(&oldmailbox, msgno, &record)) continue;
-	mailbox_message_get_fname(&oldmailbox, record.uid, newfnametail);
-	(void) unlink(newfname);
-    }
-    mailbox_close(&newmailbox);
-    mailbox_close(&oldmailbox);
+    /* Ugh, we'd have to back all the way out at this point. */
+    /* xxx we leak space for now if something failed. */
+
     return r;
 }
 
@@ -2519,6 +2555,7 @@ mailbox_sync(const char *oldname, const char *oldpath, const char *oldacl,
     }
     if (r) goto fail;
 
+    mailbox_close(&oldmailbox);
     if (mailboxp) {
 	*mailboxp = newmailbox;
     } else {
