@@ -40,7 +40,7 @@
  *
  */
 /*
- * $Id: mboxlist.c,v 1.184 2002/04/08 21:01:04 rjs3 Exp $
+ * $Id: mboxlist.c,v 1.197 2002/06/20 16:36:13 rjs3 Exp $
  */
 
 #include <config.h>
@@ -108,7 +108,7 @@ static int mboxlist_changequota(const char *name, int matchlen, int maycreate,
 static int mboxlist_getpath(const char *partition, const char *name, 
 			    char **pathp)
 {
-    int partitionlen;
+    size_t partitionlen;
     char optionbuf[MAX_MAILBOX_NAME+1];
     static char pathresult[MAX_MAILBOX_PATH];
     const char *root;
@@ -320,8 +320,11 @@ int mboxlist_update(char *name, int flags, const char *part, const char *acl)
 /*
  * Check/set up for mailbox creation
  */
+/* xxx shouldn't we be using mbtype or getting rid of it entirely? */
 static int
-mboxlist_mycreatemailboxcheck(char *name, int mbtype, char *partition, 
+mboxlist_mycreatemailboxcheck(char *name,
+			      int new_mbtype __attribute__((unused)),
+			      char *partition, 
 			      int isadmin, char *userid, 
 			      struct auth_state *auth_state, 
 			      char **newacl, char **newpartition,
@@ -339,7 +342,8 @@ mboxlist_mycreatemailboxcheck(char *name, int mbtype, char *partition,
     char *parentacl = NULL;
     unsigned long parentpartitionlen = 0;
     unsigned long parentacllen = 0;
-
+    int mbtype;
+    
     /* Check for invalid name/partition */
     if (partition && strlen(partition) > MAX_PARTITION_LEN) {
 	return IMAP_PARTITION_UNKNOWN;
@@ -357,16 +361,20 @@ mboxlist_mycreatemailboxcheck(char *name, int mbtype, char *partition,
     }
 
     /* Check to see if new mailbox exists */
-    r = mboxlist_mylookup(name, NULL, &path, NULL, &acl, tid, RMW);
+    r = mboxlist_mylookup(name, &mbtype, &path, NULL, &acl, tid, RMW);
     switch (r) {
     case 0:
-	r = IMAP_MAILBOX_EXISTS;
+	if(mbtype & MBTYPE_RESERVE)
+	    r = IMAP_MAILBOX_RESERVED;
+	else
+	    r = IMAP_MAILBOX_EXISTS;
 	
 	/* Lie about error if privacy demands */
 	if (!isadmin && 
 	    !(cyrus_acl_myrights(auth_state, acl) & ACL_LOOKUP)) {
 	    r = IMAP_PERMISSION_DENIED;
 	}
+
 	return r;       
 	break;
     case IMAP_MAILBOX_NONEXISTENT:
@@ -533,7 +541,6 @@ int mboxlist_createmailbox(char *name, int mbtype, char *partition,
     char *acl = NULL;
     const char *root = NULL;
     char *newpartition = NULL;
-    struct mailbox newmailbox;
     struct txn *tid = NULL;
     mupdate_handle *mupdate_h = NULL;
     char *mboxent = NULL;
@@ -560,7 +567,8 @@ int mboxlist_createmailbox(char *name, int mbtype, char *partition,
 	goto done;
     }
 
-    if(mbtype & MBTYPE_MOVING) {
+    /* You can't explicitly create a MOVING or RESERVED mailbox */
+    if(mbtype & (MBTYPE_MOVING | MBTYPE_RESERVE)) {
 	r = IMAP_MAILBOX_NOTSUPPORTED;
 	goto done;
     }
@@ -580,6 +588,7 @@ int mboxlist_createmailbox(char *name, int mbtype, char *partition,
     }
 
     /* 4. Create mupdate reservation */
+    /* xxx this is network I/O being done while holding a mboxlist lock */
     if (config_mupdate_server && !localonly) {
 	r = mupdate_connect(config_mupdate_server, NULL, &mupdate_h, NULL);
 	if(r) {
@@ -630,10 +639,7 @@ int mboxlist_createmailbox(char *name, int mbtype, char *partition,
 			   ((mbtype & MBTYPE_NETNEWS) ?
 			    MAILBOX_FORMAT_NETNEWS :
 			    MAILBOX_FORMAT_NORMAL), 
-			   &newmailbox);
-	if (!r) {
-	    mailbox_close(&newmailbox);
-	}
+			   NULL);
     }
 
     if (r) { /* CREATE failed */ 
@@ -713,8 +719,10 @@ int mboxlist_createmailbox(char *name, int mbtype, char *partition,
 }
 
 /* insert an entry for the proxy */
-int mboxlist_insertremote(const char *name, int mbtype, const char *host,
-			  const char *acl, void **rettid)
+/* xxx rettid needs usage? */
+int mboxlist_insertremote(const char *name, int mbtype,
+			  const char *host, const char *acl,
+			  void **rettid __attribute__((unused)))
 {
     char *mboxent;
     int r = 0;
@@ -758,7 +766,7 @@ int mboxlist_insertremote(const char *name, int mbtype, const char *host,
  */
 int mboxlist_deletemailbox(const char *name, int isadmin, char *userid, 
 			   struct auth_state *auth_state, int checkacl,
-			   int local_only)
+			   int local_only, int force)
 {
     int r;
     char *acl;
@@ -772,12 +780,14 @@ int mboxlist_deletemailbox(const char *name, int isadmin, char *userid,
     int deleteright = get_deleteright();
     mupdate_handle *mupdate_h = NULL;
 
+    if(!isadmin && force) return IMAP_PERMISSION_DENIED;
+
  retry:
     /* Check for request to delete a user:
        user.<x> with no dots after it */
     if (!strncmp(name, "user.", 5) && !strchr(name+5, '.')) {
 	/* Can't DELETE INBOX (your own inbox) */
-	if (!strcmp(name+5, userid)) {
+	if (userid && !strcmp(name+5, userid)) {
 	    r = IMAP_MAILBOX_NOTSUPPORTED;
 	    goto done;
 	}
@@ -786,7 +796,7 @@ int mboxlist_deletemailbox(const char *name, int isadmin, char *userid,
 	if (!isadmin) { r = IMAP_PERMISSION_DENIED; goto done; }
     }
 
-    r = mboxlist_mylookup(name, &mbtype, &path, NULL, &acl, &tid, 1);
+    r = mboxlist_mylookup(name, &mbtype, &path, NULL, &acl, NULL, 1);
     switch (r) {
     case 0:
 	break;
@@ -796,12 +806,18 @@ int mboxlist_deletemailbox(const char *name, int isadmin, char *userid,
 	break;
 
     default:
-	DB->abort(mbdb, tid);
 	goto done;
     }
 
     isremote = mbtype & MBTYPE_REMOTE;
 
+    /* are we reserved? (but for remote mailboxes this is okay, since
+     * we don't touch their data files at all) */
+    if(!isremote && (mbtype & MBTYPE_RESERVE) && !force) {
+	r = IMAP_MAILBOX_RESERVED;
+	goto done;
+    }
+
     /* check if user has Delete right (we've already excluded non-admins
      * from deleting a user mailbox) */
     if(checkacl) {
@@ -815,14 +831,34 @@ int mboxlist_deletemailbox(const char *name, int isadmin, char *userid,
 	    /* Lie about error if privacy demands */
 	    r = (isadmin || (access & ACL_LOOKUP)) ?
 		IMAP_PERMISSION_DENIED : IMAP_MAILBOX_NONEXISTENT;
-	    DB->abort(mbdb, tid);
 	    goto done;
 	}
     }
 
-    /* remove from mupdate */
-    /* xxx this can lead to inconsistancies if the later stuff fails */
-    if (!r && !isremote && !local_only && config_mupdate_server) {
+    /* Lock the mailbox if it isn't a remote mailbox */
+    if(!r && !isremote) {
+	r = mailbox_open_locked(name, path, acl, 0, &mailbox, 0);
+	if(r && !force) goto done;
+    }
+    
+    /* delete entry */
+    r = DB->delete(mbdb, name, strlen(name), &tid, 0);
+    switch (r) {
+    case CYRUSDB_OK: /* success */
+	break;
+    case CYRUSDB_AGAIN:
+	goto retry;
+    default:
+	syslog(LOG_ERR, "DBERROR: error deleting %s: %s",
+	       name, cyrusdb_strerror(r));
+	r = IMAP_IOERROR;
+	if(!force) goto done;
+    }
+
+    /* remove from mupdate - this can be weird if the commit below fails */
+    /* xxx this is network I/O being done while holding a mboxlist lock */
+    if ((!r || force)
+	&& !isremote && !local_only && config_mupdate_server) {
 	/* delete the mailbox in MUPDATE */
 	r = mupdate_connect(config_mupdate_server, NULL, &mupdate_h, NULL);
 	if(r) {
@@ -839,34 +875,20 @@ int mboxlist_deletemailbox(const char *name, int isadmin, char *userid,
 	mupdate_disconnect(&mupdate_h);
     }
 
-    /* delete entry */
-    r = DB->delete(mbdb, name, strlen(name), &tid, 0);
-    switch (r) {
-    case CYRUSDB_OK: /* success */
-	break;
-    case CYRUSDB_AGAIN:
-	goto retry;
-    default:
-	syslog(LOG_ERR, "DBERROR: error deleting %s: %s",
-	       name, cyrusdb_strerror(r));
-	r = IMAP_IOERROR;
-	goto done;
-    }
-
     /* commit db operations */
-    if (!r) {
+    if (!r || force) {
 	r = DB->commit(mbdb, tid);
 	if (r) {
 	    syslog(LOG_ERR, "DBERROR: failed on commit: %s",
 		   cyrusdb_strerror(r));
 	    r = IMAP_IOERROR;
 	}
+	tid = NULL;
     }
 
-    if (r || (isremote && !local_only)) goto done;
+    if ((r && !force) || isremote) goto done;
 
-    if (!r) r = mailbox_open_header_path(name, path, acl, 0, &mailbox, 0);
-    if (!r) r = mailbox_delete(&mailbox, deletequotaroot);
+    if (!r || force) r = mailbox_delete(&mailbox, deletequotaroot);
 
     /*
      * See if we have to remove mailbox's quota root
@@ -876,6 +898,13 @@ int mboxlist_deletemailbox(const char *name, int isadmin, char *userid,
     }
 
  done:
+    if(r && tid && !force) {
+	/* Abort the transaction if it is still in progress */
+	DB->abort(mbdb, tid);
+    } else if(tid && force) {
+	DB->commit(mbdb, tid);
+    }
+
     return r;
 }
 
@@ -883,18 +912,17 @@ int mboxlist_deletemailbox(const char *name, int isadmin, char *userid,
  * Rename/move a single mailbox (recursive renames are handled at a
  * higher level)
  *
- * 1. start transaction
+ * 1. get path & acl from mboxlist (with lock)
  * 2. verify acls
- * 3. open mupdate connection if needed
- * 4. Delete entry from berkeley db
- * 5. mupdate make the new entry
- * 7. delete from disk
- * 8. commit transaction
- * 9. set new mupdate entry commited
- * 10. delete old mupdate entry
- * */
-
-/* note: partition moving should really be moved to another function */
+ * 3. open mupdate connection / reserve destination if needed
+ * 4. Reserve mailbox locally (MBTYPE_RESERVE)
+ * 5. lock original mailbox / unlock mboxlist
+ * 6. Copy mailbox from src->dest
+ * 7. Perform update on mboxlist
+ * 8. finish mupdate updates (activate new, delete old)
+ * 9. delete from disk
+ *
+ */
 int mboxlist_renamemailbox(char *oldname, char *newname, char *partition, 
 			   int isadmin, char *userid, 
 			   struct auth_state *auth_state)
@@ -906,6 +934,8 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
     int mbtype;
     char *oldpath = NULL;
     char newpath[MAX_MAILBOX_PATH];
+    int oldopen = 0, newopen = 0, newreserved = 0;
+    struct mailbox oldmailbox;
     struct mailbox newmailbox;
     char *oldacl = NULL, *newacl = NULL;
     const char *root = NULL;
@@ -918,7 +948,7 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
     int madenew = 0;
 
  retry:
-    /* lookup the mailbox to make sure it exists and get its acl */
+    /* 1. get path & acl from mboxlist */
     r = mboxlist_mylookup(oldname, &mbtype, &oldpath, NULL, &oldacl, &tid, 1);
     switch (r) {
     case 0:
@@ -929,11 +959,16 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
 	goto done;
     }
 
+    if(mbtype & MBTYPE_RESERVE) {
+	r = IMAP_MAILBOX_RESERVED;
+	goto done;
+    }
+
     /* make a copy of the old ACL so it doesn't get overwritten
        by another call to mboxlist_mylookup() */
     newacl = xstrdup(oldacl);
 
-    /* Check ability to delete old mailbox */
+    /* 2. verify acls */
     if (!strcmp(oldname, newname) && !(mbtype & MBTYPE_REMOTE)) {
 	/* Attempt to move mailbox across partition */
 	if (!isadmin) {
@@ -1018,51 +1053,74 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
 	}
     }
 
-    if (!r && !partitionmove && config_mupdate_server) {
-	/* Reserve new name in MUPDATE */
-	char buf[MAX_PARTITION_LEN + HOSTNAME_SIZE + 2];
-	sprintf(buf, "%s!%s", config_servername, newpartition);
-	
+    /* 3. Open mupdate connection and reserve new name (if needed) */ 
+    if(!r && config_mupdate_server) {	
 	r = mupdate_connect(config_mupdate_server, NULL, &mupdate_h, NULL);
 	if(r) {
 	    syslog(LOG_ERR,
-		   "can not connect to mupdate server for reservation on '%s'",
+		   "can not connect to mupdate server for rename of '%s'",
 		   newname);
 	    goto done;
 	}
-	
-	/* reserve the mailbox in MUPDATE */
-	r = mupdate_reserve(mupdate_h, newname, buf);
-	if(r) {
-	    syslog(LOG_ERR,
-		   "MUPDATE: can't reserve mailbox entry for '%s'",
-		   newname);
-	    goto done;
-	}
-	
-	madenew = 1;
-    } else if(!r && partitionmove && config_mupdate_server) {
-	/* commit the update to MUPDATE */
-	char buf[MAX_PARTITION_LEN + HOSTNAME_SIZE + 2];
-	sprintf(buf, "%s!%s", config_servername, newpartition);
+	if (!partitionmove) {
+	    /* Reserve new name in MUPDATE */
+	    char buf[MAX_PARTITION_LEN + HOSTNAME_SIZE + 2];
+	    sprintf(buf, "%s!%s", config_servername, newpartition);
 
-	r = mupdate_connect(config_mupdate_server, NULL, &mupdate_h, NULL);
-	if(r) {
-	    syslog(LOG_ERR,
-		   "can not connect to mupdate server for reservation on '%s'",
-		   newname);
-	    goto done;
-	}
-	
-	r = mupdate_activate(mupdate_h, oldname, buf, newacl);
-	if(r) {
-	    syslog(LOG_ERR,
-		   "MUPDATE: can't update mailbox entry for '%s'",
-		   oldname);
-	    goto done;
+	    r = mupdate_reserve(mupdate_h, newname, buf);
+	    if(r) {
+		syslog(LOG_ERR,
+		       "MUPDATE: can't reserve mailbox entry for '%s'",
+		       newname);
+		goto done;
+	    }
+	    
+	    madenew = 1;
 	}
     }
-    
+
+    /* 4. mark as reserved in the DB */
+    if(!r && !partitionmove) {
+	mboxent = mboxlist_makeentry(mbtype | MBTYPE_RESERVE,
+				     newpartition, newacl);
+
+	r = DB->store(mbdb, newname, strlen(newname), 
+		      mboxent, strlen(mboxent), &tid);
+
+	free(mboxent);
+    }
+        
+    /* 4. unlock mboxlist, Lock oldname/oldpath */
+    if(r) {
+	syslog(LOG_ERR, "Could not lock mailbox %s during rename", oldname);
+	goto done;
+    } else {
+	DB->commit(mbdb, tid);
+	tid = NULL;
+	newreserved = 1;
+    }
+
+    if(!r) {
+	r = mailbox_open_locked(oldname, oldpath, oldacl, auth_state,
+				&oldmailbox, 0);
+	oldopen = 1;
+    }
+
+    /* 5. Copy mailbox */
+    if (!r && !(mbtype & MBTYPE_REMOTE)) {
+	/* Rename the actual mailbox */
+	assert(root != NULL); /* from above */
+	mailbox_hash_mbox(newpath, root, newname);
+	
+	r = mailbox_rename_copy(&oldmailbox, newname, newpath,
+				NULL, NULL, &newmailbox);
+	if (r) {
+	    goto done;
+	} else {
+	    newopen = 1;
+	}
+    }
+
     if (!isusermbox) {
 	/* 4. Delete entry from berkeley db */
 	r = DB->delete(mbdb, oldname, strlen(oldname), &tid, 0);
@@ -1076,6 +1134,7 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
 	    syslog(LOG_ERR, "DBERROR: error deleting %s: %s",
 		   oldname, cyrusdb_strerror(r));
 	    r = IMAP_IOERROR;
+	    mailbox_close(&newmailbox);
 	    goto done;
 	    break;
 	}
@@ -1099,29 +1158,31 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
 	goto done;
     }
 
-  done: /* ALL DATABASE OPERATIONS DONE; NEED TO DO FILESYSTEM OPERATIONS */
-    if (!r && !(mbtype & MBTYPE_REMOTE)) {
-	/* Rename the actual mailbox */
-	assert(root != NULL); /* from above */
-	mailbox_hash_mbox(newpath, root, newname);
-	
-	r = mailbox_rename(oldname, oldpath, newacl, newname, 
-			   newpath, isusermbox, NULL, NULL, &newmailbox);
-	if (!r) {
-	    mailbox_close(&newmailbox);
-	}
-    }
+    r = mailbox_rename_finish(&oldmailbox,&newmailbox,isusermbox);
 
+  done:
     if (r != 0) {
 	int r2 = 0;
 	
-	if (tid) r2 = DB->abort(mbdb, tid);
+	if (tid) {
+	    r2 = DB->abort(mbdb, tid);
+	    tid = NULL;
+	}
 	if (r2) {
 	    syslog(LOG_ERR, "DBERROR: can't abort: %s", cyrusdb_strerror(r2));
 	}
 	
+	if(newreserved) {
+	    /* remove the RESERVED mailbox entry if we failed */
+	    r2 = DB->delete(mbdb, newname, strlen(newname), NULL, 0);
+	    if(r2) {
+		syslog(LOG_ERR,
+		       "DBERROR: can't remove RESERVE entry for %s (%s)",
+		       newname, cyrusdb_strerror(r2));
+	    }
+	}
+	
 	/* unroll mupdate operations if necessary */
-	/* xxx where do we unroll if 'partitionmove == 1' ? */
 	if (madenew && config_mupdate_server) {
 	    r2 = mupdate_delete(mupdate_h, newname);
 	    if(r2 > 0) {
@@ -1155,8 +1216,9 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
 	}
     }
 
-    if (!r && !partitionmove && config_mupdate_server) {
+    if (!r && config_mupdate_server) {
 	/* commit the mailbox in MUPDATE */
+	/* This is okay even if we are moving partitions */
 	char buf[MAX_PARTITION_LEN + HOSTNAME_SIZE + 2];
 	sprintf(buf, "%s!%s", config_servername, newpartition);
 	
@@ -1197,6 +1259,9 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
 	}
     }
 
+    if(oldopen) mailbox_close(&oldmailbox);
+    if(newopen) mailbox_close(&newmailbox);
+
     if(config_mupdate_server) mupdate_disconnect(&mupdate_h);
 
     /* free memory */
@@ -1213,14 +1278,12 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
  * pointer, removes the ACL entry for 'identifier'.   'isadmin' is
  * nonzero if user is a mailbox admin.  'userid' is the user's login id.
  *
- *
  * 1. Start transaction
  * 2. Check rights
- * 3. Open mupdate connection if necessary
- * 4. Set db entry
- * 5. Change on disk
- * 6. Commit transaction
- * 7. Change mupdate entry 
+ * 3. Set db entry
+ * 4. Change backup copy (cyrus.header)
+ * 5. Commit transaction
+ * 6. Change mupdate entry 
  *
  */
 int mboxlist_setacl(char *name, char *identifier, char *rights, 
@@ -1233,6 +1296,7 @@ int mboxlist_setacl(char *name, char *identifier, char *rights,
     int mode = ACL_MODE_SET;
     int isusermbox = 0;
     struct mailbox mailbox;
+    int mailbox_open = 0;
     char *acl, *newacl = NULL;
     char *partition, *path;
     char *mboxent = NULL;
@@ -1246,117 +1310,117 @@ int mboxlist_setacl(char *name, char *identifier, char *rights,
 	isusermbox = 1;
     }
 
- retry:
+    /* 1. Start Transaction */
     /* lookup the mailbox to make sure it exists and get its acl */
-    r = mboxlist_mylookup(name, &mbtype, &path, &partition, &acl, &tid, 1);
-    switch (r) {
-    case 0:
-	break;
-    case IMAP_AGAIN:
-	goto retry;
-    default:
-	goto done;
-    }
+    do {
+	r = mboxlist_mylookup(name, &mbtype, &path, &partition, &acl, &tid, 1);
+    } while(r == IMAP_AGAIN);    
 
-    /* Can't do this to an in-transit mailbox */
-    if(mbtype & MBTYPE_MOVING) {
+    /* Can't do this to an in-transit or reserved mailbox */
+    if(!r && mbtype & (MBTYPE_MOVING | MBTYPE_RESERVE)) {
 	r = IMAP_MAILBOX_NOTSUPPORTED;
-	goto done;
     }
 
+    /* if it is not a remote mailbox, we need to unlock the mailbox list,
+     * lock the mailbox, and re-lock the mailboxes list */
+    /* we must do this to obey our locking rules */
+    if (!r && !(mbtype & MBTYPE_REMOTE)) {
+	DB->abort(mbdb, tid);
+	tid = NULL;
+
+	/* open & lock mailbox header */
+        r = mailbox_open_header_path(name, path, acl, NULL, &mailbox, 0);
+	if (!r) {
+	    mailbox_open = 1;
+	    r = mailbox_lock_header(&mailbox);
+	} 
+
+	if(!r) {
+	    do {
+		/* lookup the mailbox to make sure it exists and get its acl */
+		r = mboxlist_mylookup(name, &mbtype, &path,
+				      &partition, &acl, &tid, 1);
+	    } while( r == IMAP_AGAIN );
+	}
+
+	if(r) goto done;
+    }
+
+    /* 2. Check Rights */
     if (!r && !isadmin && !isusermbox) {
 	access = cyrus_acl_myrights(auth_state, acl);
 	if (!(access & ACL_ADMIN)) {
 	    r = (access & ACL_LOOKUP) ?
-	      IMAP_PERMISSION_DENIED : IMAP_MAILBOX_NONEXISTENT;
+		IMAP_PERMISSION_DENIED : IMAP_MAILBOX_NONEXISTENT;
 	    goto done;
 	}
     }
 
-    /* Make change to ACL */
-    newacl = xstrdup(acl);
-    if (rights) {
-	mode = ACL_MODE_SET;
-	if (*rights == '+') {
-	    rights++;
-	    mode = ACL_MODE_ADD;
-	}
-	else if (*rights == '-') {
-	    rights++;
-	    mode = ACL_MODE_REMOVE;
-	}
-	
-	if (cyrus_acl_set(&newacl, identifier, mode, cyrus_acl_strtomask(rights),
-		    isusermbox ? mboxlist_ensureOwnerRights : 0,
-		    (void *)userid))
-	{
-	    r = IMAP_INVALID_IDENTIFIER;
-	    goto done;
-	}
-    }
-    else {
-	if (cyrus_acl_remove(&newacl, identifier,
-		       isusermbox ? mboxlist_ensureOwnerRights : 0,
-		       (void *)userid)) {
-	  r = IMAP_INVALID_IDENTIFIER;
-	  goto done;
-	}
-    }
-
-    /* ok, make the change */
-    mboxent = mboxlist_makeentry(mbtype, partition, newacl);
-
-    r = DB->store(mbdb, name, strlen(name), mboxent, strlen(mboxent), &tid);
-    switch (r) {
-    case 0:
-	break;
-    case CYRUSDB_AGAIN:
-	goto retry;
-    default:
-	syslog(LOG_ERR, "DBERROR: error updating acl %s: %s",
-	       name, cyrusdb_strerror(r));
-	r = IMAP_IOERROR;
-	goto done;
-    }
-
-    if (!(mbtype & MBTYPE_REMOTE)) {
-	/* open & lock mailbox header */
-        r = mailbox_open_header_path(name, path, acl, NULL, &mailbox, 0);
-
-	if (!r) {
-	    r = mailbox_lock_header(&mailbox);
-	    if (!r) {
-		/* set it in the /var/spool part */
-		(void) mailbox_write_header(&mailbox);
+    /* 3. Set DB Entry */
+    if(!r) {
+	/* Make change to ACL */
+	newacl = xstrdup(acl);
+	if (rights) {
+	    mode = ACL_MODE_SET;
+	    if (*rights == '+') {
+		rights++;
+		mode = ACL_MODE_ADD;
+	    }
+	    else if (*rights == '-') {
+		rights++;
+		mode = ACL_MODE_REMOVE;
+	    }
+	    
+	    if (cyrus_acl_set(&newacl, identifier, mode,
+			      cyrus_acl_strtomask(rights),
+			      isusermbox ? mboxlist_ensureOwnerRights : 0,
+			      (void *)userid)) {
+		r = IMAP_INVALID_IDENTIFIER;
+	    }
+	} else {
+	    if (cyrus_acl_remove(&newacl, identifier,
+				 isusermbox ? mboxlist_ensureOwnerRights : 0,
+				 (void *)userid)) {
+		r = IMAP_INVALID_IDENTIFIER;
 	    }
-	    mailbox_close(&mailbox);
 	}
     }
 
-  done:
-    if (mboxent) free(mboxent);
+    if(!r) {
+	/* ok, change the database */
+	mboxent = mboxlist_makeentry(mbtype, partition, newacl);
 
-    if (r) {
-	int r2;
-
-	if ((r2 = DB->abort(mbdb, tid)) != 0) {
-	    syslog(LOG_ERR, "DBERROR: error aborting txn: %s",
-		   cyrusdb_strerror(r2));
-	    r2 = IMAP_IOERROR;
-	}
-    } else {
-	/* commit now */
-	switch (r = DB->commit(mbdb, tid)) {
-	case 0: 
-	    break;
-	default:
-	    syslog(LOG_ERR, "DBERROR: failed on commit: %s",
-		   cyrusdb_strerror(r));
+	do {
+	    r = DB->store(mbdb, name, strlen(name),
+			  mboxent, strlen(mboxent), &tid);
+	} while(r == CYRUSDB_AGAIN);
+    
+	if(r) {
+	    syslog(LOG_ERR, "DBERROR: error updating acl %s: %s",
+		   name, cyrusdb_strerror(r));
 	    r = IMAP_IOERROR;
 	}
     }
 
-    /* 7. Change mupdate entry  */
+    /* 4. Change backup copy (cyrus.header) */
+    /* we already have it locked from above */
+    if (!r && !(mbtype & MBTYPE_REMOTE)) {
+	if(mailbox.acl) free(mailbox.acl);
+	mailbox.acl = xstrdup(newacl);
+	r = mailbox_write_header(&mailbox);
+    }
+
+    /* 5. Commit transaction */
+    if (!r) {
+	if((r = DB->commit(mbdb, tid)) != 0) {
+	    syslog(LOG_ERR, "DBERROR: failed on commit: %s",
+		   cyrusdb_strerror(r));
+	    r = IMAP_IOERROR;
+	}
+	tid = NULL;
+    }
+
+    /* 6. Change mupdate entry  */
     if (!r && config_mupdate_server) {
         mupdate_handle *mupdate_h = NULL;
 	/* commit the update to MUPDATE */
@@ -1378,6 +1442,19 @@ int mboxlist_setacl(char *name, char *identifier, char *rights,
 	}
 	mupdate_disconnect(&mupdate_h);
     }
+
+  done:
+    if (r && tid) {
+	/* if we are mid-transaction, abort it! */
+	int r2 = DB->abort(mbdb, tid);
+	if(r2) {
+	    syslog(LOG_ERR,
+		   "DBERROR: error aborting txn in mboxlist_setacl: %s",
+		   cyrusdb_strerror(r2));
+	}
+    }
+    if (mailbox_open) mailbox_close(&mailbox);
+    if (mboxent) free(mboxent);
     if (newacl) free(newacl);
     
     return r;
@@ -1876,11 +1953,12 @@ int mboxlist_findall_alt(struct namespace *namespace,
 /*
  * Set the quota on or create a quota root
  */
-int mboxlist_setquota(const char *root, int newquota)
+int mboxlist_setquota(const char *root, int newquota, int force)
 {
     char quota_path[MAX_MAILBOX_PATH];
     char pattern[MAX_MAILBOX_PATH];
     struct quota quota;
+    int have_mailbox = 1;
     int r, t;
 
     if (!root[0] || root[0] == '.' || strchr(root, '/')
@@ -1912,10 +1990,16 @@ int mboxlist_setquota(const char *root, int newquota)
     /* look for a top-level mailbox in the proposed quotaroot */
     r = mboxlist_detail(quota.root, &t, NULL, NULL, NULL, NULL);
     if (r) {
-	return r;
+	/* are we going to force the create anyway? */
+	if(!force) return r;
+	else {
+	    have_mailbox = 0;
+	    t = 0;
+	}
     }
-    /* Can't set quota on a remote mailbox */
-    if (t & (MBTYPE_REMOTE | MBTYPE_MOVING)) {
+
+    if(t & (MBTYPE_REMOTE | MBTYPE_MOVING)) {
+	/* Can't set quota on a remote mailbox */
 	return IMAP_MAILBOX_NOTSUPPORTED;
     }
 
@@ -1933,7 +2017,8 @@ int mboxlist_setquota(const char *root, int newquota)
     strcat(pattern, ".*");
     
     /* top level mailbox */
-    mboxlist_changequota(quota.root, 0, 0, &quota);
+    if(have_mailbox)
+	mboxlist_changequota(quota.root, 0, 0, &quota);
     /* submailboxes - we're using internal names here */
     mboxlist_findall(NULL, pattern, 1, 0, 0, mboxlist_changequota, &quota);
     
@@ -2191,8 +2276,7 @@ void mboxlist_open(char *fname)
     mboxlist_dbopen = 1;
 }
 
-void
-mboxlist_close(void)
+void mboxlist_close(void)
 {
     int r;
 
