@@ -40,7 +40,7 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
  * 
- * $Id: mboxlist.h,v 1.29 2002/04/04 22:22:39 rjs3 Exp $
+ * $Id: mboxlist.h,v 1.33 2002/05/29 16:49:16 rjs3 Exp $
  */
 
 #ifndef INCLUDED_MBOXLIST_H
@@ -61,7 +61,10 @@ extern struct db *mbdb;
 
 /* flags for types of mailboxes */
 #define MBTYPE_REMOTE 0x01 /* Not on this server (part is remote host) */
-#define MBTYPE_RESERVE 0x02 /* Unused */
+#define MBTYPE_RESERVE 0x02 /* Reserved [mupdate/imapd] /
+			       Rename Target [imapd] (part is normal, but
+			       you are not allowed to create this mailbox,
+			       even though it doesn't actually exist */
 #define MBTYPE_NETNEWS 0x04 /* Netnews Mailbox */
 #define MBTYPE_MOVING 0x08 /* Mailbox in mid-transfer (part is remotehost!localpart) */
 
@@ -111,9 +114,11 @@ int mboxlist_createmailbox(char *name, int mbtype, char *partition,
 /* setting local_only disables any communication with the mupdate server
  * and deletes the mailbox from the filesystem regardless of if it is
  * MBTYPE_REMOTE or not */
+/* force ignores errors and just tries to wipe the mailbox off the face of
+ * the planet */
 int mboxlist_deletemailbox(const char *name, int isadmin, char *userid, 
 			   struct auth_state *auth_state, int checkacl,
-			   int local_only);
+			   int local_only, int force);
 
 /* Rename/move a mailbox (hierarchical) */
 int mboxlist_renamemailbox(char *oldname, char *newname, char *partition, 
@@ -163,7 +168,7 @@ int mboxlist_changesub(const char *name, const char *userid,
 char *mboxlist_hash_usersubs(const char *userid);
 
 /* set or create quota root */
-int mboxlist_setquota(const char *root, int newquota);
+int mboxlist_setquota(const char *root, int newquota, int force);
 int mboxlist_unsetquota(const char *root);
 
 /* returns a malloc() string that is the representation in the mailboxes 
