@@ -40,7 +40,7 @@
  */
 
 /*
- * $Id: pop3d.c,v 1.119 2002/04/10 18:02:42 leg Exp $
+ * $Id: pop3d.c,v 1.122 2002/06/03 18:22:27 rjs3 Exp $
  */
 #include <config.h>
 
@@ -248,10 +248,20 @@ static void popd_reset(void)
 	popd_mailbox = 0;
     }
 
-    prot_flush(popd_out);
-    if (popd_in) prot_free(popd_in);
-    if (popd_out) prot_free(popd_out);
+    if (popd_in) {
+	prot_NONBLOCK(popd_in);
+	prot_fill(popd_in);
+	
+	prot_free(popd_in);
+    }
+
+    if (popd_out) {
+	prot_flush(popd_out);
+	prot_free(popd_out);
+    }
+    
     popd_in = popd_out = NULL;
+
     close(0);
     close(1);
     close(2);
@@ -337,11 +347,12 @@ int service_init(int argc __attribute__((unused)),
 	fatal(error_message(r), EC_CONFIG);
     }
 
-    while ((opt = getopt(argc, argv, "C:sk")) != EOF) {
+    while ((opt = getopt(argc, argv, "C:Dsk")) != EOF) {
 	switch(opt) {
 	case 'C': /* alt config file - handled by service::main() */
 	    break;
-
+	case 'D': /* external debugger - handled by service::main() */
+ 	    break;
 	case 's': /* pop3s (do starttls right away) */
 	    pop3s = 1;
 	    if (!tls_enabled("pop3")) {
@@ -480,10 +491,22 @@ void shut_down(int code)
     }
     mboxlist_close();
     mboxlist_done();
+
+    if (popd_in) {
+	prot_NONBLOCK(popd_in);
+	prot_fill(popd_in);
+	prot_free(popd_in);
+    }
+
+    if (popd_out) {
+	prot_flush(popd_out);
+	prot_free(popd_out);
+    }
+
 #ifdef HAVE_SSL
     tls_shutdown_serverengine();
 #endif
-    if (popd_out) prot_flush(popd_out);
+
     exit(code);
 }
 
@@ -849,7 +872,6 @@ static void cmd_starttls(int pop3s)
     result=tls_init_serverengine("pop3",
 				 5,        /* depth to verify */
 				 !pop3s,   /* can client auth? */
-				 0,        /* require client to auth? */
 				 !pop3s);  /* TLS only? */
 
     if (result == -1) {
