@@ -40,7 +40,7 @@
  */
 
 /*
- * $Id: pop3proxyd.c,v 1.38 2002/04/23 18:41:33 rjs3 Exp $
+ * $Id: pop3proxyd.c,v 1.42 2002/06/12 20:00:12 rjs3 Exp $
  */
 #include <config.h>
 
@@ -155,8 +155,71 @@ static struct
     char *authid;
 } saslprops = {NULL,NULL,0,NULL};
 
+/* should we allow users to proxy?  return SASL_OK if yes,
+   SASL_BADAUTH otherwise */
+static int mysasl_authproc(sasl_conn_t *conn,
+			   void *context __attribute__((unused)),
+			   const char *requested_user, unsigned rlen,
+			   const char *auth_identity, unsigned alen,
+			   const char *def_realm __attribute__((unused)),
+			   unsigned urlen __attribute__((unused)),
+			   struct propctx *propctx __attribute__((unused)))
+{
+    const char *val;
+    struct auth_state *authstate;
+    int userisadmin = 0;
+    char *realm;
+
+    /* check if remote realm */
+    if ((realm = strchr(auth_identity, '@'))!=NULL) {
+	realm++;
+	val = config_getstring("loginrealms", "");
+	while (*val) {
+	    if (!strncasecmp(val, realm, strlen(realm)) &&
+		(!val[strlen(realm)] || isspace((int) val[strlen(realm)]))) {
+		break;
+	    }
+	    /* not this realm, try next one */
+	    while (*val && !isspace((int) *val)) val++;
+	    while (*val && isspace((int) *val)) val++;
+	}
+	if (!*val) {
+	    sasl_seterror(conn, 0, "cross-realm login %s denied",
+			  auth_identity);
+	    return SASL_BADAUTH;
+	}
+    }
+
+    authstate = auth_newstate(auth_identity, NULL);
+
+    /* ok, is auth_identity an admin? */
+    userisadmin = authisa(authstate, "imap", "admins");
+
+    if (alen != rlen || strncmp(auth_identity, requested_user, alen)) {
+	/* we want to authenticate as a different user; we'll allow this
+	   if we're an admin or if we've allowed ACL proxy logins */
+	if (userisadmin ||
+	    authisa(authstate, "imap", "proxyservers")) {
+
+	    /* proxy ok! */
+	    auth_freestate(authstate);
+	    return SASL_OK;
+	} else {
+	    sasl_seterror(conn, 0, "user %s is not allowed to proxy",
+			  auth_identity);
+
+	    auth_freestate(authstate);
+
+	    return SASL_BADAUTH;
+	}
+    }
+
+    return SASL_OK;
+}
+
 static struct sasl_callback mysasl_cb[] = {
     { SASL_CB_GETOPT, &mysasl_config, NULL },
+    { SASL_CB_PROXY_POLICY, &mysasl_authproc, NULL },
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
@@ -221,11 +284,12 @@ int service_main(int argc, char **argv, char **envp __attribute__((unused)))
     popd_in = prot_new(0, 0);
     popd_out = prot_new(1, 1);
 
-    while ((opt = getopt(argc, argv, "C:sk")) != EOF) {
+    while ((opt = getopt(argc, argv, "C:Dsk")) != EOF) {
 	switch(opt) {
 	case 'C': /* alt config file - handled by service::main() */
 	    break;
-
+	case 'D': /* ext. debugger - handled by service::main() */
+ 	    break;
 	case 's': /* pop3s (do starttls right away) */
 	    pop3s = 1;
 	    if (!tls_enabled("pop3")) {
@@ -335,10 +399,24 @@ void shut_down(int code)
     proc_cleanup();
     mboxlist_close();
     mboxlist_done();
+
+    if (popd_in) {
+	prot_NONBLOCK(popd_in);
+	prot_fill(popd_in);
+	
+	prot_free(popd_in);
+    }
+
+    if (popd_out) {
+	prot_flush(popd_out);
+
+	prot_free(popd_out);
+    }
+
 #ifdef HAVE_SSL
     tls_shutdown_serverengine();
 #endif
-    if (popd_out) prot_flush(popd_out);
+
     exit(code);
 }
 
@@ -561,7 +639,6 @@ static void cmd_starttls(int pop3s)
     result=tls_init_serverengine("pop3",
 				 5,        /* depth to verify */
 				 !pop3s,   /* can client auth? */
-				 0,        /* require client to auth? */
 				 !pop3s);  /* TLSv1 only? */
 
     if (result == -1) {
@@ -982,8 +1059,6 @@ void cmd_auth(char *arg)
     {
 	sleep(3);      
 
-	reset_saslconn(&popd_saslconn);
-	
 	/* convert the sasl error code to a string */
 	prot_printf(popd_out, "-ERR [AUTH] authenticating: %s\r\n",
 		    sasl_errstring(sasl_result, NULL, NULL));
@@ -995,6 +1070,8 @@ void cmd_auth(char *arg)
 	    syslog(LOG_NOTICE, "badlogin: %s %s",
 		   popd_clienthost, sasl_errdetail(popd_saslconn));
 	}
+
+	reset_saslconn(&popd_saslconn);
 	
 	return;
     }
