@@ -1,5 +1,5 @@
 /* seen_db.c -- implementation of seen database using per-user berkeley db
-   $Id: seen_db.c,v 1.28 2002/03/29 00:03:57 rjs3 Exp $
+   $Id: seen_db.c,v 1.34 2002/06/09 16:13:41 leg Exp $
  
  * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -463,7 +463,12 @@ int seen_delete_user(const char *user)
 
     /* erp! */
     r = unlink(fname);
-    if (r < 0) {
+    if (r < 0 && errno == ENOENT) {
+	syslog(LOG_WARNING, "can not unlink %s: %m", fname);
+	/* but maybe the user just never read anything? */
+	r = 0;
+    }
+    else if (r < 0) {
 	syslog(LOG_ERR, "error unlinking %s: %m", fname);
 	r = IMAP_IOERROR;
     }
@@ -544,3 +549,102 @@ int seen_reconstruct(struct mailbox *mailbox,
     return 0;
 }
 
+struct seen_merge_rock 
+{
+    struct db *db;
+    struct txn *tid;
+};
+
+/* Look up the unique id in the tgt file, if it is there, compare the
+ * last change times, and ensure that the tgt database uses the newer of
+ * the two */
+static int seen_merge_cb(void *rockp,
+			 const char *key, int keylen,
+			 const char *tmpdata, int tmpdatalen) 
+{
+    int r;
+    struct seen_merge_rock *rockdata = (struct seen_merge_rock *)rockp;
+    struct db *tgtdb = rockdata->db;
+    const char *tgtdata;
+    int tgtdatalen, dirty = 0;
+
+    if(!tgtdb) return IMAP_INTERNAL;
+
+    r = DB->fetchlock(tgtdb, key, keylen, &tgtdata, &tgtdatalen,
+		      &(rockdata->tid));
+    if(!r && tgtdata) {
+	/* compare timestamps */
+	int version, tmplast, tgtlast;
+	char *p;
+	const char *tmp = tmpdata, *tgt = tgtdata;
+	
+	/* get version */
+	version = strtol(tgt, &p, 10); tgt = p;
+	assert(version == SEEN_VERSION);
+       	/* skip lastread */
+	strtol(tgt, &p, 10); tgt = p;
+	/* skip lastuid */
+	strtol(tgt, &p, 10); tgt = p;
+	/* get lastchange */
+	tgtlast = strtol(tgt, &p, 10);
+
+	/* get version */
+	version = strtol(tmp, &p, 10); tmp = p;
+	assert(version == SEEN_VERSION);
+       	/* skip lastread */
+	strtol(tmp, &p, 10); tmp = p;
+	/* skip lastuid */
+	strtol(tmp, &p, 10); tmp = p;
+	/* get lastchange */
+	tmplast = strtol(tmp, &p, 10);
+
+	if(tmplast > tgtlast) dirty = 1;
+    } else {
+	dirty = 1;
+    }
+    
+    if(dirty) {
+	/* write back data from new entry */
+	return DB->store(tgtdb, key, keylen, tmpdata, tmpdatalen,
+			 &(rockdata->tid));
+    } else {
+	return 0;
+    }
+}
+
+static int seen_merge_p(void *rockp __attribute__((unused)),
+			const char *key __attribute__((unused)),
+			int keylen __attribute__((unused)),
+			const char *data __attribute__((unused)),
+			int datalen __attribute__((unused)))
+{
+    return 1;
+}
+
+int seen_merge(const char *tmpfile, const char *tgtfile) 
+{
+    int r = 0;
+    struct db *tmp = NULL, *tgt = NULL;
+    struct seen_merge_rock rock;
+
+    r = DB->open(tmpfile, &tmp);
+    if(r) goto done;
+	    
+    r = DB->open(tgtfile, &tgt);
+    if(r) goto done;
+
+    rock.db = tgt;
+    rock.tid = NULL;
+    
+    r = DB->foreach(tmp, "", 0, seen_merge_p, seen_merge_cb, &rock, &rock.tid);
+
+    if(r) DB->abort(rock.db, rock.tid);
+    else DB->commit(rock.db, rock.tid);
+
+ done:
+
+    if(tgt) DB->close(tgt);
+    if(tmp) DB->close(tmp);
+    
+    return r;
+}
