@@ -52,10 +52,9 @@
 * DESCRIPTION
 *	This module is the interface between Cyrus Imapd and the OpenSSL library.
 *	As of now only one filedescriptor can be handled, so only one
-*	TLS channel can be open at a time. This can be accpted, as smtpd
-*	and smtp are seperate services run by seperate programs.
+*       TLS channel can be open at a time.
 *
-*	tls_init_serverengine() is called once when smtpd is started
+*       tls_init_serverengine() is called once when the client is started
 *	in order to initialize as much of the TLS stuff as possible.
 *	The certificate handling is also decided during the setup phase,
 *	so that a peer specific handling is not possible.
@@ -94,7 +93,7 @@
 *
 */
 
-/* $Id: tls.c,v 1.35 2002/03/01 17:53:25 ken3 Exp $ */
+/* $Id: tls.c,v 1.38 2002/06/03 16:59:57 ken3 Exp $ */
 
 #include <config.h>
 
@@ -563,7 +562,6 @@ static int tls_rand_init(void)
 int     tls_init_serverengine(const char *ident,
 			      int verifydepth,
 			      int askcert,
-			      int requirecert,
 			      int tlsonly)
 {
     int     off = 0;
@@ -574,6 +572,7 @@ int     tls_init_serverengine(const char *ident,
     const char   *CAfile;
     const char   *s_cert_file;
     const char   *s_key_file;
+    int    requirecert;
     int    timeout;
 
     if (tls_serverengine)
@@ -695,6 +694,10 @@ int     tls_init_serverengine(const char *ident,
     verify_depth = verifydepth;
     if (askcert!=0)
 	verify_flags |= SSL_VERIFY_PEER | SSL_VERIFY_CLIENT_ONCE;
+
+    sprintf(buf, "tls_%s_require_cert", ident);
+    requirecert = config_getswitch(buf,
+				   config_getswitch("tls_require_cert", 0));
     if (requirecert)
 	verify_flags |= SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT
 	    | SSL_VERIFY_CLIENT_ONCE;
