@@ -1,28 +1,48 @@
 /* auth_krb.c -- Kerberos authorization
- $Id: auth_krb.c,v 1.29 1999/11/16 01:02:12 leg Exp $
+ $Id: auth_krb.c,v 1.36 2001/11/27 02:25:02 ken3 Exp $
  
- #        Copyright 1998 by Carnegie Mellon University
- #
- #                      All Rights Reserved
- #
- # Permission to use, copy, modify, and distribute this software and its
- # documentation for any purpose and without fee is hereby granted,
- # provided that the above copyright notice appear in all copies and that
- # both that copyright notice and this permission notice appear in
- # supporting documentation, and that the name of CMU not be
- # used in advertising or publicity pertaining to distribution of the
- # software without specific, written prior permission.
- #
- # CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
- # ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
- # CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
- # ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- # WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
- # ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
- # SOFTWARE.
+ * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
  */
 
+#include <config.h>
+#include <stdlib.h>
 #include <limits.h>
 #include <stdio.h>
 #include <ctype.h>
@@ -44,6 +64,8 @@
 #include "xmalloc.h"
 #include "auth.h"
 
+const char *auth_method_desc = "krb";
+
 #ifndef KRB_MAPNAME
 #define KRB_MAPNAME "/etc/krb.equiv"
 #endif
@@ -61,10 +83,10 @@ static struct auth_state auth_anonymous = {
 
 
 
-static int parse_krbequiv_line P((const char *src,
-				  char *principal, char *localuser));
-char *auth_map_krbid P((const char *real_aname, const char *real_inst,
-			const char *real_realm));
+static int parse_krbequiv_line (const char *src,
+				  char *principal, char *localuser);
+char *auth_map_krbid (const char *real_aname, const char *real_inst,
+		      const char *real_realm);
 
 /*
  * Determine if the user is a member of 'identifier'
@@ -93,7 +115,7 @@ const char *identifier;
     if (strcmp(auth_state->userid, "anonymous") == 0) return 0;
 
     aname[0] = inst[0] = realm[0] = '\0';
-    if (kname_parse(aname, inst, realm, identifier) != 0) {
+    if (kname_parse(aname, inst, realm, (char *) identifier) != 0) {
 	return 0;
     }
 
@@ -226,8 +248,9 @@ const char *real_realm;
  * Returns a pointer to a static buffer containing the canonical form
  * or NULL if 'identifier' is invalid.
  */
-char *auth_canonifyid(identifier)
+char *auth_canonifyid(identifier, len)
 const char *identifier;
+size_t len;
 {
     static char retbuf[MAX_K_NAME_SZ+1];
     char aname[ANAME_SZ];
@@ -235,21 +258,34 @@ const char *identifier;
     char realm[REALM_SZ];
     char lrealm[REALM_SZ];
     char krbhst[256];
+    char *canon_buf;
     char *p;
-    
-    if (strcasecmp(identifier, "anonymous") == 0) {
+
+    if(!len) len = strlen(identifier);
+
+    canon_buf = malloc(len + 1);
+    if(!canon_buf) return 0;
+    memcpy(canon_buf, identifier, len);
+    canon_buf[len] = '\0';
+   
+    if (strcasecmp(canon_buf, "anonymous") == 0) {
+	free(canon_buf);
 	return "anonymous";
     }
-    if (strcasecmp(identifier, "anybody") == 0 ||
-	strcasecmp(identifier, "anyone") == 0) {
+    if (strcasecmp(canon_buf, "anybody") == 0 ||
+	strcasecmp(canon_buf, "anyone") == 0) {
+	free(canon_buf);
 	return "anyone";
     }
-    
+
     aname[0] = inst[0] = realm[0] = '\0';
-    if (kname_parse(aname, inst, realm, identifier) != 0) {
+    if (kname_parse(aname, inst, realm, canon_buf) != 0) {
+	free(canon_buf);
 	return 0;
     }
 
+    free(canon_buf);
+
     /* Upcase realm name */
     for (p = realm; *p; p++) {
 	if (islower(*p)) *p = toupper(*p);
@@ -266,7 +302,7 @@ const char *identifier;
     }
 
     /* Check for krb.equiv remappings. */
-    if (p = auth_map_krbid(aname, inst, realm)) {
+    if ((p = auth_map_krbid(aname, inst, realm)) ) {
         strcpy(retbuf, p);
         return retbuf;
     }
@@ -296,14 +332,14 @@ const char *cacheid;
 {
     struct auth_state *newstate;
 
-    identifier = auth_canonifyid(identifier);
+    identifier = auth_canonifyid(identifier, 0);
     if (!identifier) return 0;
 
     newstate = (struct auth_state *)xmalloc(sizeof(struct auth_state));
 
     strcpy(newstate->userid, identifier);
     newstate->aname[0] = newstate->inst[0] = newstate->realm[0] = '\0';
-    kname_parse(newstate->aname, newstate->inst, newstate->realm, identifier);
+    kname_parse(newstate->aname, newstate->inst, newstate->realm, (char *) identifier);
 
     return newstate;
 }
