@@ -1,4 +1,5 @@
 /* +++Date last modified: 05-Jul-1997 */
+/* $Id: hash.c,v 1.8 2002/06/18 16:40:21 rjs3 Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -8,7 +9,9 @@
 #include <stdlib.h>
 
 #include "hash.h"
+#include "mpool.h"
 #include "xmalloc.h"
+#include "exitcodes.h"
 
 /*
 ** public domain code by Jerry Coffin, with improvements by HenkJan Wolthuis.
@@ -23,32 +26,46 @@
 **  - use xmalloc() and xstrdup()
 **  - cleaned up free_hash_table(), doesn't use enumerate anymore
 **  - added 'rock' to hash_enumerate()
+**
+** Further modified by Rob Siemborski.
+**  - xmalloc can never return NULL, so don't worry about it
+**  - sort the buckets for faster searching
+**  - actually, we'll just use a memory pool for this sucker
+**    (atleast, in the cases where it is advantageous to do so)
 */
 
-
 /* Initialize the hash_table to the size asked for.  Allocates space
 ** for the correct number of pointers and sets them to NULL.  If it
 ** can't allocate sufficient memory, signals error by setting the size
 ** of the table to 0.
 */
 
-hash_table *construct_hash_table(hash_table *table, size_t size)
+hash_table *construct_hash_table(hash_table *table, size_t size, int use_mpool)
 {
-      size_t i;
-      bucket **temp;
+      if(!table)
+	  fatal("construct_hash_table called without a starting table",
+		EC_TEMPFAIL);
+      if(!size)
+	  fatal("construct_hash_table called without a size", EC_TEMPFAIL);
 
-      table -> size  = size;
-      table -> table = (bucket * *)xmalloc(sizeof(bucket *) * size);
-      temp = table -> table;
+      table->size  = size;
 
-      if ( temp == NULL )
-      {
-            table -> size = 0;
-            return table;
+      /* Allocate the table -- different for using memory pools and not */
+      if(use_mpool) {
+	  /* Allocate an initial memory pool for 32 byte keys + the hash table
+	   * + the buckets themselves */
+	  table->pool =
+	      new_mpool(size * (32 + sizeof(bucket*) + sizeof(bucket)));
+	  table->table =
+	      (bucket **)mpool_malloc(table->pool,sizeof(bucket *) * size);
+      } else {
+	  table->pool = NULL;
+	  table->table = xmalloc(sizeof(bucket *) * size);
       }
+       
+      /* Allocate the table and initilize it */
+      memset(table->table, 0, sizeof(bucket *) * size);
 
-      for (i=0;i<size;i++)
-            temp[i] = NULL;
       return table;
 }
 
@@ -58,7 +75,7 @@ hash_table *construct_hash_table(hash_table *table, size_t size)
 ** sufficient for most purposes.
 */
 
-unsigned hash(char *string)
+unsigned hash(const char *string)
 {
       unsigned ret_val = 0;
       int i;
@@ -82,21 +99,24 @@ unsigned hash(char *string)
 void *hash_insert(char *key, void *data, hash_table *table)
 {
       unsigned val = hash(key) % table->size;
-      bucket *ptr;
+      bucket *ptr, *newptr;
+      bucket **prev;
 
       /*
       ** NULL means this bucket hasn't been used yet.  We'll simply
       ** allocate space for our new bucket and put our data there, with
       ** the table pointing at it.
       */
-
-      if (NULL == (table->table)[val])
+      if (!((table->table)[val]))
       {
+	  if(table->pool) {
+            (table->table)[val] =
+		(bucket *)mpool_malloc(table->pool, sizeof(bucket));
+            (table->table)[val] -> key = mpool_strdup(table->pool, key);
+	  } else {
             (table->table)[val] = (bucket *)xmalloc(sizeof(bucket));
-            if (NULL==(table->table)[val])
-                  return NULL;
-
             (table->table)[val] -> key = xstrdup(key);
+	  }
             (table->table)[val] -> next = NULL;
             (table->table)[val] -> data = data;
             return (table->table)[val] -> data;
@@ -106,33 +126,48 @@ void *hash_insert(char *key, void *data, hash_table *table)
       ** This spot in the table is already in use.  See if the current string
       ** has already been inserted, and if so, increment its count.
       */
-
-      for (ptr = (table->table)[val];NULL != ptr; ptr = ptr -> next)
-            if (0 == strcmp(key, ptr->key))
-            {
-                  void *old_data;
-
-                  old_data = ptr->data;
-                  ptr -> data = data;
-                  return old_data;
-            }
+      for (prev = &((table->table)[val]), ptr=(table->table)[val];
+	   ptr;
+	   prev=&(ptr->next),ptr=ptr->next) {
+	  int cmpresult = strcmp(key,ptr->key);
+	  if (!cmpresult) {
+	      /* Match! Replace this value and return the old */
+	      void *old_data;
+	      
+	      old_data = ptr->data;
+	      ptr -> data = data;
+	      return old_data;
+	  } else if (cmpresult < 0) {
+	      /* The new key is smaller than the current key--
+	       * insert a node and return this data */
+	      if(table->pool) {
+		  newptr = (bucket *)mpool_malloc(table->pool, sizeof(bucket));
+		  newptr->key = mpool_strdup(table->pool, key);
+	      } else {
+		  newptr = (bucket *)xmalloc(sizeof(bucket));
+		  newptr->key = xstrdup(key);
+	      }
+	      newptr->data = data;
+	      newptr->next = ptr;
+	      *prev = newptr;
+	      return data;
+	  }
+      }
 
       /*
-      ** This key must not be in the table yet.  We'll add it to the head of
-      ** the list at this spot in the hash table.  Speed would be
-      ** slightly improved if the list was kept sorted instead.  In this case,
-      ** this code would be moved into the loop above, and the insertion would
-      ** take place as soon as it was determined that the present key in the
-      ** list was larger than this one.
+      ** This key is the largest one so far.  Add it to the end
+      ** of the list (*prev should be correct)
       */
-
-      ptr = (bucket *)xmalloc(sizeof(bucket));
-      if (NULL==ptr)
-            return 0;
-      ptr -> key = xstrdup(key);
-      ptr -> data = data;
-      ptr -> next = (table->table)[val];
-      (table->table)[val] = ptr;
+      if(table->pool) {
+	  newptr=(bucket *)mpool_malloc(table->pool,sizeof(bucket));
+	  newptr->key = mpool_strdup(table->pool,key);
+      } else {
+	  newptr=(bucket *)xmalloc(sizeof(bucket));
+	  newptr->key = xstrdup(key);
+      }
+      newptr->data = data;
+      newptr->next = NULL;
+      *prev = newptr;
       return data;
 }
 
@@ -142,18 +177,21 @@ void *hash_insert(char *key, void *data, hash_table *table)
 ** the key is not in the table.
 */
 
-void *hash_lookup(char *key, hash_table *table)
+void *hash_lookup(const char *key, hash_table *table)
 {
       unsigned val = hash(key) % table->size;
       bucket *ptr;
 
-      if (NULL == (table->table)[val])
+      if (!(table->table)[val])
             return NULL;
 
       for ( ptr = (table->table)[val];NULL != ptr; ptr = ptr->next )
       {
-            if (0 == strcmp(key, ptr -> key ) )
-                  return ptr->data;
+	  int cmpresult = strcmp(key, ptr->key);
+	  if (!cmpresult)
+	      return ptr->data;
+	  else if(cmpresult < 0) /* key < ptr->key -- we passed it */
+	      return NULL;
       }
       return NULL;
 }
@@ -162,14 +200,15 @@ void *hash_lookup(char *key, hash_table *table)
 ** Delete a key from the hash table and return associated
 ** data, or NULL if not present.
 */
-
+/* Warning: use this function judiciously if you are using memory pools,
+ * since it will leak memory until you get rid of the entire hash table */
 void *hash_del(char *key, hash_table *table)
 {
       unsigned val = hash(key) % table->size;
       void *data;
       bucket *ptr, *last = NULL;
 
-      if (NULL == (table->table)[val])
+      if (!(table->table)[val])
             return NULL;
 
       /*
@@ -184,41 +223,48 @@ void *hash_del(char *key, hash_table *table)
             NULL != ptr;
             last = ptr, ptr = ptr->next)
       {
-            if (0 == strcmp(key, ptr -> key))
-            {
-                  if (last != NULL )
-                  {
-                        data = ptr -> data;
-                        last -> next = ptr -> next;
-                        free(ptr->key);
-                        free(ptr);
-                        return data;
-                  }
-
-                  /*
-                  ** If 'last' still equals NULL, it means that we need to
-                  ** delete the first node in the list. This simply consists
-                  ** of putting our own 'next' pointer in the array holding
-                  ** the head of the list.  We then dispose of the current
-                  ** node as above.
-                  */
-
-                  else
-                  {
-                        data = ptr->data;
-                        (table->table)[val] = ptr->next;
-                        free(ptr->key);
-                        free(ptr);
-                        return data;
-                  }
-            }
+	  int cmpresult = strcmp(key, ptr->key);
+	  if (!cmpresult)
+	  {
+	      if (last != NULL )
+	      {
+		  data = ptr -> data;
+		  last -> next = ptr -> next;
+		  if(!table->pool) {
+		      free(ptr->key);
+		      free(ptr);
+		  }
+		  return data;
+	      }
+	      
+	      /*
+	      ** If 'last' still equals NULL, it means that we need to
+	      ** delete the first node in the list. This simply consists
+	      ** of putting our own 'next' pointer in the array holding
+	      ** the head of the list.  We then dispose of the current
+	      ** node as above.
+	      */
+	      
+	      else
+	      {
+		  data = ptr->data;
+		  (table->table)[val] = ptr->next;
+		  if(!table->pool) {
+		      free(ptr->key);
+		      free(ptr);
+		  }
+		  return data;
+	      }
+	  } else if (cmpresult < 0) {
+	      /* its not here! */
+	      return NULL;
+	  }
       }
 
       /*
       ** If we get here, it means we didn't find the item in the table.
       ** Signal this by returning NULL.
       */
-
       return NULL;
 }
 
@@ -235,21 +281,34 @@ void free_hash_table(hash_table *table, void (*func)(void *))
       unsigned i;
       bucket *ptr, *temp;
 
-      for (i=0;i<table->size; i++)
-      {
-	    ptr = (table->table)[i];
-	    while (ptr)
-	    {
+      /* If we have a function to free the data, apply it everywhere */
+      /* We also need to traverse this anyway if we aren't using a memory
+       * pool */
+      if(func || !table->pool) {
+	  for (i=0;i<table->size; i++)
+	  {
+	      ptr = (table->table)[i];
+	      while (ptr)
+	      {
 		  temp = ptr;
 		  ptr = ptr->next;
-		  free(temp->key);
 		  if (func)
-		        func(temp->data);
-		  free(temp);
-	    }
+		      func(temp->data);
+		  if(!table->pool) {
+		      free(temp->key);
+		      free(temp);
+		  }
+	      }
+	  }
+      }
+      
+      /* Free the main structures */
+      if(table->pool) {
+	  free_mpool(table->pool);
+	  table->pool = NULL;
+      } else {
+	  free(table->table);
       }
-
-      free(table->table);
       table->table = NULL;
       table->size = 0;
 }
@@ -290,7 +349,7 @@ void fatal(const char* s, int code)
       exit(code);
 }
 
-void printer(char *string, void *data)
+void printer(char *string, void *data, void *rock)
 {
       printf("%s: %s\n", string, (char *)data);
 }
@@ -300,14 +359,9 @@ int main(void)
       hash_table table;
 
       char *strings[] = {
-            "The first string",
-            "The second string",
-            "The third string",
-            "The fourth string",
-            "A much longer string than the rest in this example.",
-            "The last string",
-            NULL
-            };
+	  "1","2","3","4","5","A decently long string",
+	  NULL
+      };
 
       char *junk[] = {
             "The first data",
@@ -321,16 +375,25 @@ int main(void)
       int i;
       void *j;
 
-      construct_hash_table(&table,200);
+      construct_hash_table(&table,200,1);
 
       for (i = 0; NULL != strings[i]; i++ )
-            hash_insert(strings[i], junk[i], &table);
+	  hash_insert(strings[i], junk[i], &table);
 
+      for (i=0;NULL != strings[i];i++)
+      {
+	  j = hash_lookup(strings[i], &table);
+	  if (!j)
+	      printf("\nERROR: %s was not in table.",
+		     strings[i]);
+      }
+      
       for (i=0;NULL != strings[i];i++)
       {
             printf("\n");
-            hash_enumerate(&table, printer);
-            hash_del(strings[i],&table);
+            hash_enumerate(&table, printer, NULL);
+            if(!hash_del(strings[i],&table))
+		printf("ERROR WITH DELETE of '%s'\n", strings[i]);
       }
 
       for (i=0;NULL != strings[i];i++)
@@ -341,6 +404,7 @@ int main(void)
             else  printf("\nERROR: %s was deleted but is still in table.",
                   strings[i]);
       }
+      printf("\n");
       free_hash_table(&table, NULL);
       return 0;
 }
