@@ -39,7 +39,7 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: service.c,v 1.32 2002/03/08 19:26:17 leg Exp $ */
+/* $Id: service.c,v 1.37 2002/06/04 19:49:55 ken3 Exp $ */
 
 #include <config.h>
 
@@ -68,7 +68,7 @@
 
 #include "service.h"
 
-extern int optind;
+extern int optind, opterr;
 extern char *optarg;
 
 /* number of times this service has been used */
@@ -138,6 +138,7 @@ static int libwrap_ask(struct request_info *r, int fd)
 #endif
 
 extern void config_init(const char *, const char *);
+extern const char *config_getstring(const char *key, const char *def);
 extern const char *config_dir;
 
 static int getlockfd(char *service)
@@ -257,19 +258,26 @@ int main(int argc, char **argv, char **envp)
     struct request_info request;
     int opt;
     char *alt_config = NULL;
+    int call_debugger = 0;
     int soctype;
     int typelen = sizeof(soctype);
 
-    while ((opt = getopt(argc, argv, "C:")) != EOF) {
+    opterr = 0; /* disable error reporting,
+		   since we don't know about service-specific options */
+    while ((opt = getopt(argc, argv, "C:D")) != EOF) {
 	switch (opt) {
 	case 'C': /* alt config file */
 	    alt_config = optarg;
 	    break;
+	case 'D':
+	    call_debugger = 1;
+	    break;
 	default:
 	    break;
 	}
     }
-    optind = 1;
+    opterr = 1; /* enable error reporting */
+    optind = 1; /* reset the option index for parsing by the service */
 
     p = getenv("CYRUS_VERBOSE");
     if (p) verbose = atoi(p) + 1;
@@ -291,6 +299,18 @@ int main(int argc, char **argv, char **envp)
     }
     config_init(alt_config, service);
 
+    if (call_debugger) {
+	char debugbuf[1024];
+	int ret;
+	const char *debugger = config_getstring("debug_command", NULL);
+	if (debugger) {
+	    snprintf(debugbuf, sizeof(debugbuf), debugger, 
+		     argv[0], getpid(), service);
+	    syslog(LOG_DEBUG, "running external debugger: %s", debugbuf);
+	    ret = system(debugbuf); /* run debugger */
+	    syslog(LOG_DEBUG, "debugger returned exit status: %d", ret);
+	}
+    }
     syslog(LOG_DEBUG, "executed");
 
     /* set close on exec */
