@@ -37,13 +37,14 @@
 # AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
 # OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #
+# $Id: Admin.pm,v 1.27 2002/06/12 20:04:10 rjs3 Exp $
 
 package Cyrus::IMAP::Admin;
 use strict;
 use Cyrus::IMAP;
 use vars qw($VERSION
 	    *create *delete *deleteacl *listacl *list *rename *setacl
-	    *subscribed *quota *quotaroot *info);
+	    *subscribed *quota *quotaroot *info *setinfo);
 
 $VERSION = '1.00';
 
@@ -129,7 +130,7 @@ sub reconstruct {
     my $rc;
     my $msg;
     if($recurse == 1) {
-      ($rc, $msg) = $self->send('', '', 'RECONSTRUCT "%s" RECURSEIVE',
+      ($rc, $msg) = $self->send('', '', 'RECONSTRUCT "%s" RECURSIVE',
 				$mailbox);
     } else {
       ($rc, $msg) = $self->send('', '', 'RECONSTRUCT "%s"', $mailbox);
@@ -138,7 +139,29 @@ sub reconstruct {
     if($rc eq "OK") {
       $rc = 1;
     } else {
-      $rc = 0;
+      if($self->{support_referrals} && $msg =~ m|^\[REFERRAL\s+([^\]\s]+)\]|) {
+	my ($refserver, $box) = Cyrus::IMAP->fromURL($1);
+	my $port = 143;
+	
+	if($refserver =~ /:/) {
+	  $refserver =~ /([^:]+):(\d+)/;
+	  $refserver = $1; $port = $2;
+	}
+	
+	my $cyradm = Cyrus::IMAP::Admin->new($refserver, $port)
+	  or die "cyradm: cannot connect to $refserver\n";
+	$cyradm->addcallback({-trigger => 'EOF',
+			      -callback => \&_cb_ref_eof,
+			      -rock => \$cyradm});
+	$cyradm->authenticate()
+	  or die "cyradm: cannot authenticate to $refserver\n";
+	
+	my $ret = $cyradm->reconstruct($mailbox,$recurse);
+	$cyradm = undef;
+	return $ret;
+      } else {
+	$rc = 0;
+      }
     }
     return $rc;
 }
@@ -454,8 +477,17 @@ sub listquotaroot {
 
 sub renamemailbox {
   my ($self, $src, $dest, $ptn) = @_;
+
+  $self->addcallback({-trigger => 'NO',
+		      -callback => sub {
+			print $_ . "\n";
+		      }});
+
   my ($rc, $msg) = $self->send('', '', 'RENAME %s %s%a%a', $src, $dest,
 			       $ptn ? ' ' : $ptn, $ptn);
+
+  $self->addcallback({-trigger => 'NO'});
+		    
   if ($rc eq 'OK') {
     $self->{error} = undef;
     1;
@@ -601,15 +633,14 @@ sub setquota {
   }
 }
 
-# xxx this is entirely untested ;)
 sub getinfo {
   my ($self,$box) = @_;
   my $pat;
 
-  if($box == undef) {
+  if(!defined($box)) {
     $pat = "/server/*";
   } else {
-    $pat = "/mailbox/$box/*";
+    $pat = "/mailbox/{$box}/*";
   }
 
   if(!$self->{support_annotatemore}) {
@@ -622,16 +653,15 @@ sub getinfo {
 		      -callback => sub {
 			my %d = @_;
 			my $text = $d{-text};
-			my $annotation;
 
-			if($text =~ /^\([^\)]*\)$/) {
+			if($text =~ /^\(.*\)$/) {
 			  # list of annotations
 			  $text =~ s/^\(//;
 			  
 			  while($text !~ /^\)/) {
 			    if($text =~
 			       /^\s*\"([^\"]*)\"\s+\(\"([^\"]*)\"\s+\"([^\"]*)\"\)/) {
-				 ${$d{-rock}}{$1} = $3;
+				 $d{-rock}{$1} = $3;
 				 $text =~ s/^\s*\"([^\"]*)\"\s+\(\"([^\"]*)\"\s+\"([^\"]*)\"\)//;
 			       } else {
 				 # hrm, error
@@ -639,12 +669,11 @@ sub getinfo {
 				 return undef;
 			       }
 			  }
-			} elsif ($text =~ /^\"([^\"]+)\"/) {
+			} elsif ($text =~
+			       /^\s*\"([^\"]*)\"\s+\(\"([^\"]*)\"\s+\"([^\"]*)\"\)/) {
 			  # Single annotation, but possibly multiple values
 			  # however, we are only asking for one value, so...
-			  $annotation = $1;
-			  $text =~ /\"(\S*)\"\)/;
-			  ${$d{-rock}}{$annotation} = $1;
+			  $d{-rock}{$1} = $3;
 		        } else {
 			  next;
 			}
@@ -657,7 +686,7 @@ sub getinfo {
   $self->addcallback({-trigger => 'ANNOTATION'});
   if ($rc eq 'OK') {
     $self->{error} = undef;
-    @info;
+    %info;
   } else {
     $self->{error} = $msg;
     ();
@@ -665,6 +694,27 @@ sub getinfo {
 }
 *info = *getinfo;
 
+sub setinfoserver {
+  my ($self, $entry, $value) = @_;
+
+  if(!$self->{support_annotatemore}) {
+    $self->{error} = "Remote does not support ANNOTATEMORE.";
+    return undef;
+  }
+
+  my ($rc, $msg) = $self->send('', '', "SETANNOTATION \"/server/%s\" (\"value.shared\" \"%s\")",
+			       $entry, $value);
+
+  if ($rc eq 'OK') {
+    $self->{error} = undef;
+    1;
+  } else {
+    $self->{error} = $msg;
+    undef;
+  }
+}
+*setinfo = *setinfoserver;
+
 sub error {
   my $self = shift;
   $self->{error};
