@@ -37,6 +37,8 @@
 # AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
 # OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #
+# $Id: Shell.pm,v 1.17 2002/05/25 19:57:49 leg Exp $
+#
 # A shell framework for IMAP::Cyrus::Admin
 #
 # run(*FH|'FH')
@@ -120,6 +122,9 @@ my %builtins = (exit =>
 		  [\&_sc_delete, 'mailbox [host]', 'delete mailbox'],
 		delete => 'deletemailbox',
 		dm => 'deletemailbox',
+		info =>
+		  [\&_sc_info, '[mailbox]',
+		   'display mailbox/server metadata'],
 		reconstruct =>
 		  [\&_sc_reconstruct, 'mailbox', 'reconstruct mailbox (if supported'],
 		renamemailbox =>
@@ -133,6 +138,9 @@ my %builtins = (exit =>
 		   'set ACLs on mailbox'],
 		setacl => 'setaclmailbox',
 		sam => 'setaclmailbox',
+		setinfo =>
+		  [\&_sc_setinfo, '[motd|comment] text',
+		   'set server metadata'],
 		setquota =>
 		  [\&_sc_setquota,
 		   'mailbox resource value [resource value ...]',
@@ -1121,6 +1129,62 @@ sub _sc_version {
   0;
 }
 
+sub _sc_info {
+  my ($cyrref, $name, $fh, $lfh, @argv) = @_;
+  my (@nargv, $opt);
+  shift(@argv);
+  while (defined ($opt = shift(@argv))) {
+    # gack.  bloody tcl.
+    last if $opt eq '--';
+    if ($opt =~ /^-/) {
+      die "usage: info [mailbox]\n";
+    }
+    else {
+      push(@nargv, $opt);
+      last;
+    }
+  }
+  push(@nargv, @argv);
+  if (!$cyrref || !$$cyrref) {
+    die "info: no connection to server\n";
+  }
+  my %info = $$cyrref->getinfo(@nargv);
+  if (defined $$cyrref->error) {
+    $lfh->[2]->print($$cyrref->error, "\n");
+    return 1;
+  }
+  foreach my $attrib (keys %info) {
+    $attrib =~ /([^\/]*)$/;
+    $lfh->[1]->print($1, ": ", $info{$attrib}, "\n");
+  }
+  0;
+}
+
+sub _sc_setinfo {
+  my ($cyrref, $name, $fh, $lfh, @argv) = @_;
+  my (@nargv, $opt);
+  shift(@argv);
+  while (defined ($opt = shift(@argv))) {
+    last if $opt eq '--';
+    if ($opt =~ /^-/) {
+      die "usage: setinfo [motd|comment] text\n";
+    }
+    else {
+      push(@nargv, $opt);
+      last;
+    }
+  }
+  push(@nargv, @argv);
+  if (@nargv < 2) {
+    die "usage: setinfo [motd|comment] text\n";
+  }
+  if (!$cyrref || !$$cyrref) {
+    die "setinfo: no connection to server\n";
+  }
+  $$cyrref->setinfoserver(@nargv) || die "setinfo: " . $$cyrref->error . "\n";
+  0;
+}
+
 ###############################################################################
 
 #
@@ -1253,6 +1317,10 @@ last command will be used if one is not specified.
 
 Show help for C<command> or all commands.
 
+=item C<info> [I<mailbox>]
+
+Display the mailbox/server metadata.
+
 =item listaclmailbox I<mailbox>
 
 =item listacl I<mailbox>
