@@ -39,6 +39,7 @@
 # OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #
 #
+# $Id: cyradm.sh,v 1.10 2002/05/25 19:57:49 leg Exp $
 case "x$BASH_VERSION" in
 x) exec perl -MCyrus::IMAP::Shell -e shell -- ${1+"$@"} ;;
 *) exec perl -MCyrus::IMAP::Shell -e shell -- "$@" ;;
@@ -150,11 +151,15 @@ last command will be used if one is not specified.
 
 Show help for C<command> or all commands.
 
-=item listaclmailbox I<mailbox>
+=item C<info> [I<mailbox>]
 
-=item listacl I<mailbox>
+Display the mailbox/server metadata.
 
-=item lam I<mailbox>
+=item C<listaclmailbox> I<mailbox>
+
+=item C<listacl> I<mailbox>
+
+=item C<lam> I<mailbox>
 
 List ACLs on the specified mailbox.
 
