@@ -46,24 +46,26 @@ typedef union {
 #define	CONTAINS	289
 #define	MATCHES	290
 #define	REGEX	291
-#define	OVER	292
-#define	UNDER	293
-#define	ALL	294
-#define	LOCALPART	295
-#define	DOMAIN	296
-#define	USER	297
-#define	DETAIL	298
-#define	DAYS	299
-#define	ADDRESSES	300
-#define	SUBJECT	301
-#define	MIME	302
-#define	METHOD	303
-#define	ID	304
-#define	OPTIONS	305
-#define	LOW	306
-#define	NORMAL	307
-#define	HIGH	308
-#define	MESSAGE	309
+#define	COUNT	292
+#define	VALUE	293
+#define	OVER	294
+#define	UNDER	295
+#define	ALL	296
+#define	LOCALPART	297
+#define	DOMAIN	298
+#define	USER	299
+#define	DETAIL	300
+#define	DAYS	301
+#define	ADDRESSES	302
+#define	SUBJECT	303
+#define	MIME	304
+#define	METHOD	305
+#define	ID	306
+#define	OPTIONS	307
+#define	LOW	308
+#define	NORMAL	309
+#define	HIGH	310
+#define	MESSAGE	311
 
 
 extern YYSTYPE yylval;
