@@ -1,6 +1,6 @@
 /* actions.c -- executes the commands for timsieved
  * Tim Martin
- * $Id: actions.c,v 1.25 2001/01/16 01:03:11 leg Exp $
+ * $Id: actions.c,v 1.30 2002/02/19 18:50:15 ken3 Exp $
  * 
  */
 /*
@@ -62,6 +62,8 @@
 #include <string.h>
 
 #include "prot.h"
+#include "tls.h"
+#include "util.h"
 #include "imapconf.h"
 #include "xmalloc.h"
 #include "sieve_interface.h"
@@ -93,8 +95,7 @@ int actions_init(void)
   return TIMSIEVE_OK;
 }
 
-
-int actions_setuser(char *userid)
+int actions_setuser(const char *userid)
 {
   char hash;
   char *foo=sieve_dir;
@@ -102,8 +103,7 @@ int actions_setuser(char *userid)
 
   sieve_dir=(char *) xmalloc(1024);
   
-  hash = (char) tolower((int) *userid);
-  if (!islower((int) hash)) { hash = 'q'; }
+  hash = (char) dir_hash_c(userid);
     
   snprintf(sieve_dir, 1023, "%s/%c/%s", foo, hash,userid);
 
@@ -147,7 +147,7 @@ int scriptname_valid(mystring_t *name)
 
 int capabilities(struct protstream *conn, sasl_conn_t *saslconn)
 {
-    char *sasllist;
+    const char *sasllist;
     unsigned mechcount;
 
     /* implementation */
@@ -165,7 +165,9 @@ int capabilities(struct protstream *conn, sasl_conn_t *saslconn)
     /* Sieve capabilities */
     prot_printf(conn,"\"SIEVE\" \"%s\"\r\n",sieve_listextensions());
 
-    /* TODO: STARTTLS */
+    if (tls_enabled("sieve")) {
+	prot_printf(conn, "\"STARTTLS\"\r\n");
+    }
 
     prot_printf(conn,"OK\r\n");
 
@@ -267,7 +269,8 @@ static int countscripts(char *name)
 
 
 /* save name as a sieve script */
-int putscript(struct protstream *conn, mystring_t *name, mystring_t *data)
+int putscript(struct protstream *conn, mystring_t *name, mystring_t *data,
+	      int verify_only)
 {
   FILE *stream;
   char *dataptr;
@@ -284,20 +287,26 @@ int putscript(struct protstream *conn, mystring_t *name, mystring_t *data)
       return result;
   }
 
-  /* see if this would put the user over quota */
-  maxscripts = config_getint("sieve_maxscripts",5);
+  if (verify_only)
+      stream = tmpfile();
 
-  if (countscripts(string_DATAPTR(name))+1 > maxscripts)
-  {
-    prot_printf(conn,
-		"NO (\"QUOTA\") \"You are only allowed %d scripts on this server\"\r\n",
-		maxscripts);
-    return TIMSIEVE_FAIL;
+  else {
+      /* see if this would put the user over quota */
+      maxscripts = config_getint("sieve_maxscripts",5);
+
+      if (countscripts(string_DATAPTR(name))+1 > maxscripts)
+      {
+	  prot_printf(conn,
+		      "NO (\"QUOTA\") \"You are only allowed %d scripts on this server\"\r\n",
+		      maxscripts);
+	  return TIMSIEVE_FAIL;
+      }
+
+      snprintf(path, 1023, "%s.script.NEW", string_DATAPTR(name));
+
+      stream = fopen(path, "w+");
   }
 
-  snprintf(path, 1023, "%s.script.NEW", string_DATAPTR(name));
-
-  stream = fopen(path, "w+");
   if (stream == NULL) {
       prot_printf(conn, "NO \"Unable to open script for writing (%s)\"\r\n",
 		  path);
@@ -338,8 +347,10 @@ int putscript(struct protstream *conn, mystring_t *name, mystring_t *data)
   fflush(stream);
   fclose(stream);
   
-  snprintf(p2, 1023, "%s.script", string_DATAPTR(name));
-  rename(path, p2);
+  if (!verify_only) {
+      snprintf(p2, 1023, "%s.script", string_DATAPTR(name));
+      rename(path, p2);
+  }
 
   prot_printf(conn, "OK\r\n");
 
@@ -555,7 +566,7 @@ int cmd_havespace(struct protstream *conn, mystring_t *sieve_name, unsigned long
     {
 	prot_printf(conn,
 		    "NO (\"QUOTA\") \"Script size is too large. "
-		    "Max script size is %d bytes\"\r\n",
+		    "Max script size is %ld bytes\"\r\n",
 		    maxscriptsize);
 	return TIMSIEVE_FAIL;
     }
